/**
 * @class Sb.TopMessageBox
 * <p>Utility class to display messages at the top of page.<p/>
 * <p>Example usage:</p>
 *<pre><code>
Sb.TopMessageBox.show('Editor works in Trial mode.');
</code></pre>
 * @singleton
 */
Ext.define('Sb.TopMessageBox', {
	extend: 'Ext.util.Observable',
	singleton: true,

	/**
	 * Returns a reference to element
	 * @return {Ext.Element} The element
	 */
	getEl: function () {
		return this.el;
	},

	/**
	 * Displays a message box with specific message.
	 * @param {String} msg The message box body text
	 */
	show: function (msg) {
		if (!this.el) {
			this.el = Ext.getBody().insertFirst({
				cls: 'sb-top-message-box',
				cn: {
					cls: 'sb-top-message sb-top-message-warning',
					cn: {
						cls: 'sb-top-message-text'
					}
				}
			});
			Ext.EventManager.onWindowResize(this.onResize, this);
		}

		this.el.update(msg);

		if (!this.el.isVisible()) {
			this.el.show();
		}

		this.onResize();
	},

	/**
	 * Returns true if the message box is currently displayed
	 * @return {Boolean} True if the message box is visible, else false
	 */
	isVisible: function () {
		return this.el && this.el.isVisible();
	},

	/**
	 * Hides the message box if it is displayed
	 */
	hide: function () {
		if (this.isVisible()) {
			this.el.hide();
			this.onResize();
		}
	},

	onResize: function () {
		var w = this.isVisible() ? this.el.getWidth() : 0;
		var h = this.isVisible() ? this.el.getHeight() : 0;
		this.fireEvent('resize', this, w, h);
	}
});