/**
 * @class Sb.WidgetTypeManager
 * <p>Manages types of widgets, when they are not created yet, helps in buttons and DnD.</p>
 * @singleton
 */
Ext.define('Sb.WidgetTypeManager', function () {
	var types = new Ext.util.MixedCollection(false,
		function (widgetType) {
			return widgetType.getKey();
		}
	);

	return {
		singleton: true,

		/**
		 * Initialize types and clean them, if we already have them
		 */
		init: function () {
			types.each(function (type) {
				type.init();
			});
		},

		/**
		 * Widget factory by type.
		 *
		 * @param {Object} config The configuration options for the widget model
		 * @return {Sb.model.Widget}
		 */
		create: function (config) {
			var widget, type = config.type.camelize(true) + 'Widget';
			if (Sb.model[type]) {
				widget = new Sb.model[type](config);
			} else {
				widget = new Sb.model.Widget(config);
			}

			return widget;
		},

		/**
		 * Creates type once and return it
		 * @param {String} key Name of the type
		 * @return {Sb.widgetType.Base}
		 */
		get: function (key) {
			key = key.toString().camelize();
			var typeName = key.toUpperCaseFirst();
			if (!types.map[key]) {
				if (!Sb.widgetType[typeName]) {
					typeName = 'Base';
				}
				types.add(
					new Sb.widgetType[typeName](key)
				);
			}
			return types.map[key];
		},

		/**
		 * Register for type it`s component(-s)
		 *
		 * @param {Ext.Component} component
		 * @param {Sb.widgetType.Base} widgetType
		 */
		registerComponent: function (component, widgetType) {
			this.get(widgetType.toString()).components.add(component);
		},

		/**
		 * Toggle all registered components to state according current availability for each widget types
		 */
		toggleButtons: function () {
			types.each(function (type) {
				type.toggleButton();
			});
		}
	};
});