/**
 * @class Sb.auxilaryModel.Flash
 * @extends Sb.model.Node
 * @constructor
 * @param {Object} config Configuration for image
 */
Ext.define('Sb.auxilaryModel.Flash', {
	extend: 'Sb.model.Node',

	dataMceJson: null,

	setElement: function (domEl) {
		this.el = Ext.get(domEl);
		this.canUpdateEl = false;

		this.dataMceJson = Sb.utils.Tinymce.parseJson(this.el);

		var data = Ext.apply({}, this.dataMceJson);

		data = Ext.apply(data, {
			sourceUrl: this.dataMceJson.params.src,
			width: this.el.getWidth(true),
			height: this.el.getHeight(true),
			alignTo: Sb.utils.Tinymce.getAlignment(this.el)
		});

		if (!this.dataMceJson.params.originalWidth && !this.dataMceJson.params.originalHeight) {
			this.setOriginalSize({
				width: this.el.getWidth(true),
				height: this.el.getHeight(true)
			});
		}

		this.unserialize(data);

		this.set('isProportional', this.isProportional());

		this.canUpdateEl = true;
		this.setAlignment(this.get("alignTo"));

		this.fireEvent('load', this);
		return true;
	},

	onUnserializePropertyChange: function (key) {
		var value = this.data[key];

		if ('sourceUrl' == key) {
			this.dataMceJson.params.src = value;
			var rawSourceFileName	= decodeURIComponent(value.split('/').pop());
			rawSourceFileName		= rawSourceFileName.replace(/=/g, '%');
			var sourceFileName		= decodeURIComponent(rawSourceFileName);
			sourceFileName			= sourceFileName.replace('\\', '');
			this.set('sourceFileName', sourceFileName);
		} else if ('alignTo' == key) {
			value = value.toLowerCase();
			if (!(/^(left|right|center)$/).test(value)) {
				value = 'none';
			}
			this.data[key] = value;
		} else {
			this.set(key, value);
		}
		this.updateDomElement(key, value);
	},

	updateDomElement: function (key, value) {
		if (!this.canUpdateEl) {
			return;
		}
		var el = this.el;
		if ('sourceUrl' == key) {
			Sb.utils.Tinymce.saveJson(el, this.dataMceJson);
		} else if ('width' == key) {
			el.set({width: value});
		} else if ('height' == key) {
			el.set({height: value});
			this.fireEvent('heightchange', this);
		} else if ('alignTo' == key) {
			this.setAlignment(value);
		}
	},

	setAlignment: function (value) {
		var el = this.el;
		var style = {
			'float': '',
			marginLeft: '',
			marginRight: '',
			display: ''
		};
		if ('left' == value) {
			style['float'] = 'left';
			style.marginRight = '20px';
		} else if ('right' == value) {
			style['float'] = 'right';
			style.marginLeft = '20px';
		} else if ('center' == value) {
			style.marginLeft = 'auto';
			style.marginRight = 'auto';
			style.display = 'block';
		}
		el.setStyle(style);
		Sb.utils.Tinymce.cacheStyleInEditor(el);
	},

	getOriginalSize: function () {
		return {
			width: this.dataMceJson.params.originalWidth,
			height: this.dataMceJson.params.originalHeight
		};
	},

	isProportional: function () {
		return Sb.Utils.isProportional(this.get('width'), this.get('height'), this.getOriginalSize());
	},

	selectInEditor: function () {
		Sb.utils.Tinymce.selectInEditor(this.el);
	},

	removeFromEditor: function () {
		Sb.utils.Tinymce.removeFromEditor(this.el);
	},

	setOriginalSize: function (size) {
		this.dataMceJson.params.originalWidth = size.width.toString();
		this.dataMceJson.params.originalHeight = size.height.toString();
		Sb.utils.Tinymce.saveJson(this.el, this.dataMceJson);
	},

	getProportionalSize: function (dimension, value) {
		return Sb.Utils.getProportionalSize(dimension, value, this.getOriginalSize());
	},

	/**
	 * Update dimension size depends of proportions constrain
	 * @param {String} dimension 'width' or 'height'
	 */
	updateDimension: function (dimension) {
		var secondDimension, value;

		if (this.get('isProportional')) {
			secondDimension = (dimension == 'width') ? 'height': 'width';
			value = this.getProportionalSize(secondDimension, this.get(secondDimension));
		} else {
			value = this.get(dimension);
		}

		this.set(dimension, value);
	},

	getFlashVars: function () {
		return this.dataMceJson.params.flashvars;
	},

	setFlashVars: function (vars) {
		if (this.dataMceJson.params.flashvars != vars) {
			this.dataMceJson.params.flashvars = vars;
			Sb.utils.Tinymce.saveJson(this.el, this.dataMceJson);
		}
	}
});