/**
 * @class Sb.controller.DesignPresets
 * @extends Sb.Controller
 *
 * Design presets controller
 */
Ext.define('Sb.controller.DesignPresets', {
	extend: 'Sb.Controller',

	init: function () {
		this.designPresetsStore = new Ext.data.DirectStore({
			storeId: 'DesignPresets',
			directFn: Sb.SiteService.getDesignPresets,
			fields: [
				'type',
				{name: 'presets', type: 'object'}
			]
		});
	},

	onLaunch: function () {
		this.stylePanel = Ext.ComponentMgr.all.find(function (component) {
			return component.xtype === 'stylepanel';
		});
		this.designPresetsPanel = Ext.ComponentMgr.all.find(function (component) {
			return component.xtype === 'designpresetspanel';
		});

		this.designPresetsPanel.on('afterrender', function () {
			Sb.HeaderManager.loadHeaders();
			this.designPresetsStore.load();
		}, this);

		this.designPresetsPanel.view.on('click', this.applyPreset, this);

		this.showStylePanel();
	},

	showStylePanel: function () {
		if (!Ext.state.Manager.get('isNotOpenDesignPresets')) {
			Ext.state.Manager.set('isNotOpenDesignPresets', true);
			Sb.Editor.Ui.mainWindow.tabs.getComponent('style').setActiveTab('presets');
		}
	},

	applyPreset: function (dataview, index) {
		this.designPresetsStore.each(function (record) {
			var type = record.get('type'),
				presets = record.get('presets');
			if (index < presets.length) {
				presets[index].design.presetType = type;
				presets[index].design.presetName = presets[index].name;
				this.unserializePreset(presets[index]);
				return false;
			}
			index -= presets.length;
		}, this);
	},

	unserializePreset: function (preset) {
		var data;

		data = Ext.apply({
			layoutWidget: Ext.apply({reInitUuid: true}, preset.layout)
		}, preset.design);

		if (preset.banner) {
			data.banner = Ext.apply(preset.banner, {
				visibilityMode: 'first'
			});
		}

		if (!Sb.Editor.model.design.unserialize(data)) {
			return;
		}

		Sb.Editor.Ui.mainWindow.statusBar.setMessageWarning(
			Sb.Locale.get('designPresetsSiteWideModulesReplacingWarning')
		);
		Sb.editor.UiContext.registerChanges();
	}
});