/**
 * @class Sb.dashboard.tile.DisqusDetailedTile
 * @extends Sb.dashboard.tile.NotificationTile
 * Class for disqus recent comments tile
 * @xtype sb.dashboard.tile.disqusdetail
 */
Ext.define('Sb.dashboard.tile.DisqusDetailedTile', {
	extend: 'Sb.dashboard.tile.NotificationTile',
	xtype: 'sb.dashboard.tile.disqusdetail',

	initComponent: function () {
		this.model = this.manager.getModel('disqus');

		this.itemsTpl = new Ext.XTemplate(
			'<tpl for="."><div{[xindex % 2 === 0 ? " class=sb-item-even" : \'\']}>' +
			'<table class="sb-comment"><tbody><tr>' +
				'<td rowspan="3" width="80"><tpl if="newPost"><div class="sb-notifications-new"></div></tpl>' +
					'<img width="70" src="{avatar}" /></td>' +
				'<td class="sb-comments-name">{username}</td>' +
				'<td align="right">{createdAt}</td>' +
			'</tr><tr>' +
				'<td rowspan="1" colspan="2">{title}</td>' +
			'</tr><tr>' +
				'<td rowspan="1" colspan="2" class="sb-comments-message">{[fm.nl2br(values.message)]}' +
				'<p><a target="_blank" href="{threadLink}">{linkToCommentLabel}</a></p></td>' +
			'</tr></tbody></table>' +
			'</div></tpl>'
		);

		if (Sb.Editor.model.get('disqusId')) {
			Ext.apply(this, {
				bbar: [{
					cls: 'sb-dashboard-manage-button',
					text: Sb.Locale.get('disqusManageComments'),
					scope: this,
					height: 45,
					handleMouseEvents: false,
					handler: function () {
						window.open('http://' + Sb.Editor.model.get('disqusId') + '.' + Sb.Config.disqusDomain);
					}
				}]
			});
		}

		this.mon(this.model, 'countnewchange', this.updateTabHeader, this);
		Sb.dashboard.tile.DisqusDetailedTile.superclass.initComponent.call(this);
	},

	pagingChangeHandler: function (toolbar, pageData) {
		if (toolbar.store.getTotalCount()) {
			var start = (pageData.activePage - 1) * toolbar.pageSize,
				range = this.model.proxy.data.posts.slice(start, start + toolbar.pageSize),
				readPost = 0,
				lastViewId = 0;

			Ext.iterate(range, function (post) {
				if (post.id > lastViewId) {
					lastViewId = post.id;
				}

				if (post.newPost) {
					readPost++;
					post.newPost = false;
				}
			});
			this.model.setCountNew(this.model.getCountNew() - readPost);

			if (lastViewId != this.model.get('lastViewId')) {
				this.model.set('lastViewId', lastViewId);
				this.model.saveSettings();
			}
		}
	},

	updateTabHeader: function () {
		var title = '<div class="sb-dashboard-tab-text"><h1>' +
			Sb.Locale.get('comments') + ' (<b>' + this.model.getCountNew() + '</b>)</h1>' +
			Sb.Locale.get('countComments', this.model.proxy.data.total) +
		'</div>';
		this.ownerCt.commentsTab.setTitle(title);
	},

	loadDataHandler: function () {
		this.doRefresh();
		this.updateTabHeader();
	},

	createStore: function () {
		return new Ext.data.JsonStore({
			proxy: this.model.proxy,
			root: 'posts',
			fields: ['avatar', 'title', 'message', 'createdAt', 'threadLink',
					'username', 'linkToCommentLabel', 'newPost']
		});
	}

});