/**
 * @class Sb.dashboard.tile.EcwidDetailedTile
 * @extends Sb.dashboard.tile.NotificationTile
 * Class for ecwid recent orders tile
 * @xtype sb.dashboard.tile.ecwiddetail
 */
Ext.define('Sb.dashboard.tile.EcwidDetailedTile', {
	extend: 'Sb.dashboard.tile.NotificationTile',
	xtype: 'sb.dashboard.tile.ecwiddetail',

	itemsPerPage: 3,

	isSettingsLoaded: false,

	initComponent: function () {
		this.model = this.manager.getModel('ecwid');

		this.itemsTpl = new Ext.XTemplate(
			'<tpl for="."><div{[xindex % 2 === 0 ? " class=sb-item-even" : \'\']}>' +
			'<table cellspacing="0" cellpadding="0"><tbody><tr>' +
				'<td><tpl if="this.isNew(number)"><div class="sb-notifications-new"></div></tpl>' +
					'<a target="_blank" href="' + Sb.Config.ecwidControlPanelUrl + '/#order=edit' + '{number}" class="sb-orders-id">#{vendorNumber}</a>' +
					' {[this.formatDate(values.created)]}</td>' +
				'<td align="right"><h2>{fulfillmentStatus}</h2></td>' +
				'<td rowspan="4" class="sb-orders-shipping">' +
					'<h2>' + Sb.Locale.get('eshopShippingAddress') + '</h2> <span>{[values.shippingPerson.street]}<br/>{[values.shippingPerson.city]}' +
					'<br/>{[values.shippingPerson.countryName]}</span>' +
					'<h2>' + Sb.Locale.get('eshopShippingPrice') + '</h2> {[Ecwid.formatCurrency(values.shippingCost)]}' +
					'<h2>Tax</h2> {[Ecwid.formatCurrency(values.taxCost)]}' +
				'</td>' +
			'</tr><tr>' +
				'<td class="sb-orders-name">{customerName}</td>' +
				'<td align="right">{[Sb.Locale.get(\'eshopContacts\', values.customerEmail, values.shippingPerson.phone)]}</td>'+
			'</tr><tr>' +
				'<td class="sb-orders-items" colspan="2"><tpl for="items"><table>' +
					'<tr><td class="sb-orders-itemname">{name} (' + Sb.Locale.get('eshopSku') + ': {sku}, ' +
					Sb.Locale.get('eshopQuantity') + ': {quantity})</td><td class="sb-orders-itemspacer"></td>' +
					'<td class="sb-orders-itemprice">{[Ecwid.formatCurrency(values.price * values.quantity)]}</td></tr>' +
				'</table></tpl></td>' +
			'</tr><tr><td colspan="2">' +
					Sb.Locale.get('eshopPaymentStatus') + ': {paymentStatus}' +
					'<p class="sb-orders-total">' + Sb.Locale.get('eshopTotal') + ': {[Ecwid.formatCurrency(values.totalCost)]}</p>' +
			'</td></tr></tbody></table>' +
			'</div></tpl>',
			{
				disableFormats: true,
				isNew: function (id) {
					return (id > this.model.settings.lastViewId);
				}.createDelegate(this),

				formatDate: function (date) {
					var createdAt = Date.parseDate(date, 'Y-m-d H:i:s');
					return createdAt.format(Sb.Config.locales[Sb.Config.locale].dateFormat + ' ' +
							Sb.Config.locales[Sb.Config.locale].timeFormat);
				}
			}
		);

		Ext.apply(this, {
			bbar: [{
				cls: 'sb-dashboard-manage-button',
				text: Sb.Locale.get('eshopManageOrders'),
				scope: this,
				height: 45,
				handleMouseEvents: false,
				handler: function () {
					var model = this.model;
					if (model.data.orders && model.data.orders.length) {
						model.set('lastViewId', model.data.orders[0].number);
						model.saveSettings();
					}
					window.open(Sb.Config.ecwidControlPanelUrl);
				}
			}]
		});

		Sb.dashboard.tile.EcwidDetailedTile.superclass.initComponent.call(this);
		this.model.loadData();
	},

	// private
	afterRender: function () {
		Sb.dashboard.tile.EcwidDetailedTile.superclass.afterRender.call(this);
		var storeId = Sb.Editor.model.get('ecwidStoreId');
		this.getBottomToolbar().setDisabled(!storeId);
		if (storeId) {
			Sb.EcwidWidgetsManager.initEcwidAPI(Sb.Editor.model.get('ecwidStoreId'), function () {
				Ecwid.OnAPILoaded.add(function () {
					this.doRefresh();
				}.createDelegate(this));
			}, this);
		}
	},

	updateTabHeader: function (total, newCount) {
		var title = '<div class="sb-dashboard-tab-text"><h1>' +
			Sb.Locale.get('orders') + ' (<b>' + newCount + '</b>)</h1>' +
			Sb.Locale.get('eshopCountOrders', total) +
		'</div>';
		this.ownerCt.ordersTab.setTitle(title);
	},

	loadDataHandler: function () {
		var model = this.model;
		if (!this.isSettingsLoaded) {
			this.mon(model, 'settingsload', function () {
				this.isSettingsLoaded = true;
				this.loadDataHandler();
			}, this, {single: true});
			model.set('lastViewId', 0);
			model.loadSettings();
			return;
		}
		var newCount = 0;
		Ext.each(model.data.orders, function (order) {
			if (order.fulfillmentStatus == 'NEW') {
				newCount++;
			}
		});
		this.updateTabHeader(model.data.orders.length, newCount);
		this.proxy.data = model.data.orders;
		if (window.Ecwid && window.Ecwid.formatCurrency) {
			this.doRefresh();
		}
	},

	createStore: function () {
		return new Ext.data.JsonStore({
			proxy: this.proxy,
			fields : [
				'customerEmail', 'customerName', 'created', 'paymentStatus', 'fulfillmentStatus',
				'subtotalCost', 'shippingCost', 'taxCost', 'totalCost', 'items', 'shippingPerson',
				'number', 'vendorNumber'
			]
		});
	}
});
