/**
 * @class Sb.dashboard.tile.UpsellTile
 * @extends Sb.dashboard.Tile
 * @constructor
 * Tile for show upsell limits
 * @xtype sb.dashboard.tile.upsell
 */
Ext.define('Sb.dashboard.tile.UpsellTile', {
	extend: 'Sb.dashboard.Tile',
	xtype: 'sb.dashboard.tile.upsell',

	initComponent: function () {
		Ext.apply(this, {
			titleString: Sb.Locale.get('resources'),
			logoCls: 'sb-dashboard-logo-upsell'
		});

		Sb.dashboard.tile.UpsellTile.superclass.initComponent.call(this);

		this.showLimits(
			Sb.Limits.getWidgetLimits(Sb.Editor.model),
			Sb.Limits.checkPages(Sb.Editor.model)
		);
	},

	showLimits: function (widgetLimits, pagesLimit) {
		var el = this.getContentEl(),
			html = Sb.Locale.get('resourcesText') + '<ul>',
			widgetName = '',
			text = '',
			isExceeded = false,
			limitExceededStyle = 'style="color: red;"',
			widgetLimitIsExceeded = false;

		if (pagesLimit.limit !== -1) {
			html += this.formatLimit(
				pagesLimit.isExceeded ? limitExceededStyle : '',
				Sb.Locale.get(
					'limitPagesText',
					pagesLimit.count,
					pagesLimit.limit
				)
			);
		}

		Ext.iterate(widgetLimits, function (widgetTypeName, data) {
			widgetName = Sb.WidgetTypeManager.get(widgetTypeName).getTranslation('title');
			isExceeded = false;

			if (Ext.isBoolean(data.limit)) {
				isExceeded = !data.limit && data.count > 0;
				text = Sb.Locale.get(
					data.limit ? 'limitAvailableText' : 'limitNotAvailableText',
					widgetName
				);
			} else {
				isExceeded = data.count > data.limit;
				text = Sb.Locale.get(
					'countableLimitText',
					data.count,
					widgetName,
					data.limit
				);
			}

			widgetLimitIsExceeded = widgetLimitIsExceeded || isExceeded;
			html += this.formatLimit(
				isExceeded ? limitExceededStyle : '',
				text
			);
		}, this);
		html += '</ul>';

		el.removeAll();
		el.add({
			html: html
		});
		if (pagesLimit.isExceeded || widgetLimitIsExceeded) {
			el.ownerCt.ownerCt.add({
				cls: 'sb-dashboard-upsell-toolbar',
				colspan: 2,
				layout: 'table',
				layoutConfig: {
					columns: 2
				},
				items: [{
					cls: 'sb-dashboard-upsell-warning-icon'
				}, {
					xtype: 'label',
					text: Sb.Locale.get('upsellHint'),
					cellCls: 'sb-dashboard-upsell-hint'
				}, {
					xtype: 'button',
					colspan: 2,
					iconCls: 'sb-dashboard-upsell-button-icon',
					text: Sb.Locale.get('publishUpsell'),
					scope: this,
					handler: function () {
						this.manager.getWindow().closeHandler();
						Sb.Editor.Ui.mainWindow.publishSiteStart();
					}
				}]
			});
		}
		el.doLayout();
	},

	formatLimit: function (style, text) {
		return String.format(
			'<li {0}>{1}</li>',
			style,
			text
		);
	}
});