/**
 * Edit anchor dialog for text widget
 *
 * @class Sb.editor.AnchorEditDialog
 * @extends Sb.Window
 */
Ext.define('Sb.editor.AnchorEditDialog', {
	extend: 'Sb.Window',

	title		: Sb.Locale.get('anchorEditDialogTitle'),
	layout		: 'fit',
	width		: 390,
	resizable	: false,
	modal		: true,

	initComponent: function () {
		Ext.apply(this, {
			defaultButton: 'titleField',
			items: [{
				xtype: 'form',
				autoHeight: true,
				labelAlign: 'top',
				border: false,
				items: [{
					xtype: 'compositefield',
					msgTarget: 'under',
					anchor: '100%',
					items: [{
						xtype: 'textfield',
						id: 'titleField',
						fieldLabel: Sb.Locale.get('anchorEditDialogTitleField'),
						flex: 1,
						maxLength: 255,
						regex: /^[a-z]+[a-z0-9\-_]*$/i,
						regexText: Sb.Locale.get('anchorInvalidName')
					}, {
						xtype: 'button',
						text: Sb.Locale.get('Remove'),
						iconCls: 'sb-delete-icon',
						scope: this,
						handler: function () {
							if (this.action == 'update') {
								var ed = this.editor,
									link = ed.dom.getParent(ed.selection.getNode(), 'A');
								ed.execCommand('mceRemoveNode', 0, link);
							}
							this.close();
						}
					}]
				}, {
					id: 'editWarning',
					html: Sb.Locale.get('anchorEditWarning'),
					border: false,
					hidden: true
				}]
			}],
			buttons: [{
				text: Sb.Locale.get('ok'),
				handler: this.onUpdate,
				scope: this
			}, {
				text: Sb.Locale.get('cancelButtonTitle'),
				handler: this.close,
				scope: this
			}]
		});
		Sb.editor.AnchorEditDialog.superclass.initComponent.call(this);
	},

	show: function (ed) {
		this.editor = ed;
		var link = ed.dom.getParent(ed.selection.getNode(), 'A'),
			value = ed.dom.getAttrib(link, 'name');

		if (value) {
			this.action = 'update';
			var titleField = Ext.getCmp('titleField');
			titleField.setValue(value);
			Ext.getCmp('editWarning').show();
		}

		Sb.editor.AnchorEditDialog.superclass.show.call(this);
	},

	onUpdate: function () {
		var ed = this.editor,
			nameField = Ext.getCmp('titleField'),
			name = nameField.getValue(),
			bookmark = Ext.isIE ? ed.lastSelectionBookmark : null;

		if (!nameField.isValid()) {
			return;
		}

		if (Ext.isIE) {
			ed.selection.moveToBookmark(bookmark);
		}

		if (this.action != 'update') {
			ed.selection.collapse(1);
		}

		var link = ed.dom.getParent(ed.selection.getNode(), 'A');
		if (link) {
			link.setAttribute('name', name);
		} else {
			ed.execCommand('mceInsertContent', 0, ed.dom.createHTML('a', {
				name: name,
				'class': 'mceItemAnchor'
			}, ''));
		}
		Sb.Editor.Ui.editComponent.updateModelFromInstance();

		this.close();
	}
});