/**
 * @class Sb.editor.DocumentManager
 * @extends Ext.DataView
 * Document manager class
 * @constructor
 * @param {Object} config The config object
 */
Ext.define('Sb.editor.DocumentManager', {
	extend: 'Ext.DataView',

	emptyText: '<div class="empty-text">' + Sb.Locale.get('uploadNoDocuments') + '</div>',

	tpl: new Ext.XTemplate(
		'<tpl for=".">',
			'<div class="thumb-wrap">',
			'<div class="thumb"><img src="', Sb.Config.skinUrl,
			'/images/icons/file_type/{[Sb.Utils.getDocumentTypeByExtension(values.extension)]}.png',
			'" alt="" title="{name}"></div>',
			'<span class="x-editable">{name:ellipsis(17)}</span></div>',
		'</tpl>',
		'<div class="x-clear"></div>'
	),

	height: 320,

	cls: 'sb-editor-documentmanager',

	overClass:'x-view-over',

	itemSelector:'div.thumb-wrap',

	multiSelect: true,

	initComponent: function () {
		this.store = this.model.store;
		Sb.editor.DocumentManager.superclass.initComponent.apply(this, arguments);
	},

	loadDocuments: function () {
		this.model.load();
	},

	/**
	 * Add a new resources to store
	 * @param resources
	 */
	addDocuments: function (resources) {
		this.model.loadData(resources, true);
	},

	/**
	 * Remove resources from store
	 */
	removeDocuments: function () {
		var selectedDocuments = this.getSelectedRecords(),
			documentsToRemove = [];
		Ext.each(selectedDocuments, function (document) {
			documentsToRemove.push(document.get('name'));
		});
		this.selectRange(0, 0);
		this.model.remove(documentsToRemove);
	}
});