/**
 * @class {Sb.editor.EmailLinkEditDialog}
 * @constructor
 * @property {Ext.form.TextField} emailField
 */
Ext.define('Sb.editor.EmailLinkEditDialog', {
	extend: 'Sb.Window',

	width: 350,
	title: Sb.Locale.get('emailLinkEditDialogTitle'),
	layout: 'fit',
	modal: true,

	initComponent: function () {
		Ext.apply(this, {
			items: {
				layout: 'form',
				labelAlign: 'top',
				autoHeight: true,
				border: false,
				items: [{
					xtype: 'textfield',
					ref: '../emailField',
					vtype: 'email',
					anchor: '100%',
					validationEvent: 'blur',
					msgTarget: 'under',
					fieldLabel: Sb.Locale.get('emailLinkEditDialogLinkField'),
					listeners: {
						scope: this,
						specialKey: function (field, e) {
							if (e.getKey() == e.ENTER) {
								e.preventDefault();
								this.okHandler();
							}
						}
					}
				}, {
					xtype: 'box',
					cls: 'sb-editor-box-description',
					html: Sb.Locale.get('emailLinkEditDialogHint')
				}]
			},
			buttons: [{
				text: Sb.Locale.get('ok'),
				handler: this.okHandler,
				scope: this
			}, {
				text: Sb.Locale.get('cancelButtonTitle'),
				handler: this.close,
				scope: this
			}]
		});
		Sb.editor.EmailLinkEditDialog.superclass.initComponent.call(this);
	},

	okHandler: function () {
		if (this.emailField.isValid()) {
			this.updateLink();
			this.close();
		}
	},

	close: function () {
		Sb.editor.EmailLinkEditDialog.superclass.close.call(this);
		this.editor.focus();
	},

	updateLink: function () {
		var href = this.emailField.getValue(),
			bookmark = Ext.isIE ? this.editor.lastSelectionBookmark : null;

		if (href === '') {
			Sb.utils.Tinymce.removeNode(this.editor, this.linkEl, bookmark);
			return;
		}

		this.editor.execCommand("mceBeginUndoLevel");

		if (!this.linkEl) {
			this.linkEl = Sb.utils.Tinymce.createLinkNode(this.editor, bookmark);
		}

		this.editor.dom.setAttrib(this.linkEl, 'href', 'mailto:' + href);
		Sb.utils.Tinymce.selectNode(this.editor, this.linkEl);

		this.editor.execCommand("mceEndUndoLevel");
	},

	/**
	 * @param {tinymce.Editor} editor
	 */
	show: function (editor) {
		var href;

		this.editor = editor;
		this.linkEl = Sb.utils.Tinymce.getLinkNodeFromSelection(editor);

		Sb.editor.EmailLinkEditDialog.superclass.show.call(this);

		href = editor.dom.getAttrib(this.linkEl, 'href') || Sb.Editor.model.get('ownerInfo').email;
		href = href.replace('mailto:', '');
		this.emailField.setRawValue(href);
		this.emailField.focus(false, 100);
	}
});