/**
 * @class Sb.editor.LayoutDialog
 * @extends Ext.Window
 * Layout editor dialog
 * @constructor
 * @param {Object} config The config object
 * @xtype layoutdialog
 */
Ext.define('Sb.editor.LayoutDialog', {
	extend: 'Ext.Window',
	xtype: 'layoutdialog',

	initComponent: function () {
		var radioCls = 'sb-layout-dialog-sidebar',
			radio1Cls = radioCls + (Sb.Editor.model.design.isSidebarsSwitched() ? '2-1hidden' : '1-2hidden'),
			radio2Cls = radioCls + (Sb.Editor.model.design.isSidebarsSwitched() ? '2hidden-1' : '1hidden-2');

		Ext.apply(this, {
			manager: Sb.Editor.windowManager.getManager(9),
			renderTo: Ext.getBody(),
			modal: true,
			constrain: true,
			draggable: false,
			resizable: false,
			closable: false,
			shadow: false,
			stateful: false,
			shim: true,
			buttonAlign: 'center',
			width: 400,
			cls: 'x-window-dlg sb-layout-dialog-switch',
			plain: true,
			footer:	true,
			fbar: [{
				iconCls: 'sb-ok-icon',
				text: Sb.Locale.get('ok'),
				scope: this,
				handler: this.okHandler
			}, {
				iconCls: 'sb-cancel-icon',
				text: Sb.Locale.get('cancelButtonTitle'),
				scope: this,
				handler: this.cancelHandler
			}],
			items : [{
				xtype: 'box',
				autoEl: 'p',
				html: Sb.Locale.get('switchFrom3To2Columns')
			}, {
				xtype: 'radiogroup',
				id:	'idSelectSideBar',
				columns: 1,
				defaults: {
					xtype: 'radio',
					name: 'selectSideBar'
				},
				items: [{
					boxLabel: Sb.Editor.model.design.isSidebarsSwitched() ?
						Sb.Locale.get('sidebar2Sidebar1Hidden') :
						Sb.Locale.get('sidebar1Sidebar2Hidden'),
					inputValue: 'hideRight',
					itemCls: radio1Cls + ' ' + radio1Cls + '-selected',
					checked: true,
					listeners: {
						check: function (radio, checked) {
							var wrapperEl = radio.getEl().up('.' + radio1Cls);
							if (checked) {
								wrapperEl.addClass(radio1Cls + '-selected');
							} else {
								wrapperEl.removeClass(radio1Cls + '-selected');
							}
						}
					}
				}, {
					boxLabel: Sb.Editor.model.design.isSidebarsSwitched() ?
						Sb.Locale.get('sidebar1Sidebar2Hidden') :
						Sb.Locale.get('sidebar2Sidebar1Hidden'),
					inputValue: 'hideLeft',
					itemCls: radio2Cls,
					checked: false,
					listeners: {
						check: function (radio, checked) {
							var wrapperEl = radio.getEl().up('.' + radio2Cls);
							if (checked) {
								wrapperEl.addClass(radio2Cls + '-selected');
							} else {
								wrapperEl.removeClass(radio2Cls + '-selected');
							}
						}
					}
				}]
			}]
		});
		Sb.editor.LayoutDialog.superclass.initComponent.apply(this, arguments);
	},

	/**
	 * Action for hide left or right sidebar after switching 3 to 2 columns
	 */
	okHandler: function () {
		var value = Ext.getCmp('idSelectSideBar').getValue().getGroupValue();
		this.fireEvent('change', value);
		this.close();
	},

	cancelHandler: function () {
		this.fireEvent('cancel');
		this.close();
	}
});