/**
 * @class Sb.editor.LayoutPanel
 * @extends Ext.Panel
 * Layout editor panel
 * @constructor
 * @param {Object} config The config object
 * @xtype layoutpanel
 */
Ext.define('Sb.editor.LayoutPanel', {
	extend: 'Ext.Panel',
	xtype: 'layoutpanel',

	initComponent: function () {
		// doll control
		this.doll = new Sb.editor.LayoutDoll({
			listeners: {
				scope: this,
				change: this.onDollChange
			}
		});
		var dollColumn = {
			width: 300,
			cls: 'sb-layout-doll-column',
			items: [this.doll, {
				cls: 'sb-layout-doll-help-arrow',
				xtype: 'box'
			}, {
				border: false,
				cls: 'sb-layout-doll-help',
				html: Sb.Locale.get('dollHelp')
			}]
		};

		// columns count control
		this.layout1ColumnRadio = new Ext.form.Radio({
			boxLabel: Sb.Locale.get('layoutSidebars1Column'),
			inputValue: 1
		});
		this.layout2ColumnsRadio = new Ext.form.Radio({
			boxLabel: Sb.Locale.get('layoutSidebars2Column'),
			inputValue: 2
		});
		this.layout3ColumnsRadio = new Ext.form.Radio({
			boxLabel: Sb.Locale.get('layoutSidebars3Column'),
			inputValue: 3
		});
		this.layoutSidebarsRadioGroup = new Ext.form.RadioGroup({
			itemCls: 'sb-layout-columns-count-control',
			fieldLabel: Sb.Locale.get('layoutSidebars'),
			defaults: {
				name: 'layoutColumnsCount'
			},
			items: [this.layout1ColumnRadio, this.layout2ColumnsRadio, this.layout3ColumnsRadio],
			listeners: {
				scope: this,
				change: this.onColumnsCountChange
			}
		});

		// columns width controls
		var layoutColumnWidthConfig = {
			width: 40,
			allowBlank: false,
			allowNegative: false,
			decimalPrecision: 0,
			enableKeyEvents: true,
			minText: Sb.Locale.get('layoutColumnWidthMoreThan'),
			minValue: 100,
			maxLength: 4,
			maxValue: 10000,
			itemCls: 'sb-layout-unit-field',
			msgTarget: 'layoutColumnsWidthMsg'
		};
		this.layoutSidebarLeftWidth = new Ext.form.NumberField(
			Ext.apply({
				listeners: {
					scope: this,
					keyup: this.onColumnLeftWidthChange,
					focus: this.onFieldFocus
				}
			}, layoutColumnWidthConfig)
		);
		this.layoutSidebarContentWidth = new Ext.form.NumberField({
			width: 40,
			decimalPrecision: 0,
			id: 'layoutColumnContentWidth',
			disabled: true
		});
		this.layoutSidebarRightWidth = new Ext.form.NumberField(
			Ext.apply({
				listeners: {
					scope: this,
					keyup: this.onColumnRightWidthChange,
					focus: this.onFieldFocus
				}
			}, layoutColumnWidthConfig)
		);

		var layoutColumnLabelConfig = {
			border: false,
			cls: 'sb-layout-field-label'
		};
		var layoutColumnLeftLabel = Ext.apply({
			html: Sb.Locale.get('layoutColumnLeftLabel')
		}, layoutColumnLabelConfig);
		var layoutColumnContentLabel = Ext.apply({
			html: Sb.Locale.get('layoutColumnContentLabel')
		}, layoutColumnLabelConfig);
		var layoutColumnRightLabel = Ext.apply({
			html: Sb.Locale.get('layoutColumnRightLabel')
		}, layoutColumnLabelConfig);

		this.layoutSidebarLeftWidthColumn = new Ext.Panel({
			items: [layoutColumnLeftLabel, this.layoutSidebarLeftWidth]
		});
		this.layoutSidebarContentWidthColumn = new Ext.Panel({
			cls: 'sb-layout-field-padding',
			items: [layoutColumnContentLabel, this.layoutSidebarContentWidth]
		});
		this.layoutSidebarRightWidthColumn = new Ext.Panel({
			cls: 'sb-layout-field-padding',
			items: [layoutColumnRightLabel, this.layoutSidebarRightWidth]
		});

		var layoutColumnsWidthFields = {
			layout: 'column',
			defaults: {
				border: false,
				columnWidth: 0.33
			},
			items: [
				this.layoutSidebarLeftWidthColumn,
				this.layoutSidebarContentWidthColumn,
				this.layoutSidebarRightWidthColumn]
		};

		// columns height controls
		var layoutColumnHeightConfig = {
			width: 40,
			allowBlank: false,
			allowNegative: false,
			decimalPrecision: 0,
			enableKeyEvents: true,
			minText: Sb.Locale.get('layoutColumnHeightMoreThan'),
			maxValue: 100000,
			minValue: 100,
			maxLength: 5,
			msgTarget: 'layoutColumnsHeightMsg'
		};

		this.layoutSidebarLeftHeight = new Ext.form.NumberField(
			Ext.apply({
				fieldLabel: Sb.Locale.get('layoutColumnLeftLabel'),
				listeners: {
					scope: this,
					keyup: this.onColumnLeftHeightChange,
					focus: this.onFieldFocus
				}
			}, layoutColumnHeightConfig)
		);
		this.layoutSidebarContentHeight = new Ext.form.NumberField(
			Ext.apply({
				fieldLabel: Sb.Locale.get('layoutColumnContentLabel'),
				listeners: {
					scope: this,
					keyup: this.onColumnContentHeightChange,
					focus: this.onFieldFocus
				}
			}, layoutColumnHeightConfig)
		);
		this.layoutSidebarRightHeight = new Ext.form.NumberField(
			Ext.apply({
				fieldLabel: Sb.Locale.get('layoutColumnRightLabel'),
				listeners: {
					scope: this,
					keyup: this.onColumnRightHeightChange,
					focus: this.onFieldFocus
				}
			}, layoutColumnHeightConfig)
		);

		this.layoutSidebarLeftHeightColumn = new Ext.Panel({
			items: [layoutColumnLeftLabel, this.layoutSidebarLeftHeight]
		});
		this.layoutSidebarContentHeightColumn = new Ext.Panel({
			cls: 'sb-layout-field-padding',
			items: [layoutColumnContentLabel, this.layoutSidebarContentHeight]
		});
		this.layoutSidebarRightHeightColumn = new Ext.Panel({
			cls: 'sb-layout-field-padding',
			items: [layoutColumnRightLabel, this.layoutSidebarRightHeight]
		});

		var layoutColumnsHeightFields = {
			layout: 'column',
			defaults: {
				border: false,
				columnWidth: 0.33
			},
			items: [
				this.layoutSidebarLeftHeightColumn,
				this.layoutSidebarContentHeightColumn,
				this.layoutSidebarRightHeightColumn
			]
		};

		// columns switch button
		this.layoutSidebarsSwitchButton = new Ext.Button({
			width: 120,
			cls: 'sb-layout-switch',
			iconCls: 'sb-switch-icon',
			text: Sb.Locale.get('switchSidebar'),
			handler: this.switchColumns,
			scope: this
		});

		this.pressColumnsToBottomCheckbox = new Ext.form.Checkbox({
			boxLabel: Sb.Locale.get('layoutPressColumnsToBottom'),
			scope: this,
			handler: this.onPressColumnsToBottomChange
		});

		// Width/heght fields column
		var layoutMsgConfig = {
				xtype: 'box',
				cls: 'x-form-invalid-msg',
				width: 'auto'
			},
			layoutColumnsHeightMsg = Ext.apply({id: 'layoutColumnsHeightMsg'}, layoutMsgConfig),
			layoutColumnsWidthMsg = Ext.apply({id: 'layoutColumnsWidthMsg'}, layoutMsgConfig),
			widthHeightFieldsColumn = {
				columnWidth: 1,
				layout: 'form',
				defaults: {
					border: false
				},
				cls: 'sb-layout-width-height-column',
				items: [
					this.layoutSidebarsRadioGroup,
					this.layoutSidebarsSwitchButton, {
						cls: 'sb-layout-width-fields-title',
						html: Sb.Locale.get('layoutColumnWidth')
					}, layoutColumnsWidthFields, layoutColumnsWidthMsg, {
						html: Sb.Locale.get('layoutColumnMinHeight')
					}, layoutColumnsHeightFields, layoutColumnsHeightMsg,
					this.pressColumnsToBottomCheckbox
				]
			};

		// layout width controls
		this.layoutFixedRadio = new Ext.form.Radio({
			name: 'fixed',
			boxLabel: Sb.Locale.get('fixed'),
			listeners: {
				scope: this,
				check: this.onLayoutTypeRadioCheck
			}
		});
		this.layoutElasticRadio = new Ext.form.Radio({
			name: 'elastic',
			boxLabel: Sb.Locale.get('elastic'),
			listeners: {
				scope: this,
				check: this.onLayoutTypeRadioCheck
			}
		});
		this.layoutWidthField = new Ext.form.NumberField({
			width: 40,
			allowBlank: false,
			allowNegative: false,
			enableKeyEvents: true,
			decimalPrecision: 0,
			maxLength: 4,
			msgTarget: 'layoutWidthMsg',
			listeners: {
				scope: this,
				keyup: this.onLayoutWidthFieldChange
			}
		});
		this.layoutWidthUnit = new Ext.BoxComponent({
			autoEl: {
				tag: 'span'
			},
			cls: 'sb-layout-unit',
			html: Sb.Locale.get('px')
		});

		var layoutWidthMsg = Ext.apply({id: 'layoutWidthMsg'}, layoutMsgConfig),
			layoutWidthColumn = {
				columnWidth: 0.5,
				defaults: {
					border: false
				},
				items: [{
					cls: 'sb-layout-layout-column-title',
					html: Sb.Locale.get('layoutWidth')
				}, {
					layout: 'column',
					defaults: {
						border: false
					},
					items: [{
						cls: 'sb-layout-fixed-radio',
						items: this.layoutFixedRadio
					}, {
						cls: 'sb-layout-fixed-width',
						items: [this.layoutWidthField, this.layoutWidthUnit]
					}, {
						cls: 'sb-layout-elastic-radio',
						items: this.layoutElasticRadio
					}]
				}, layoutWidthMsg]
			};

		//layout margins controls
		var layoutMarginsVerticalLabel = {
			xtype: 'label',
			text: Sb.Locale.get('vertical'),
			cls: 'sb-layout-margins-label'
		};
		var layoutMarginsHorizontalLabel = {
			xtype: 'label',
			text: Sb.Locale.get('horizontal'),
			cls: 'sb-layout-margins-label'
		};

		var layoutMarginsConfig = {
			allowBlank: false,
			allowNegative: false,
			decimalPrecision: 0,
			minValue: 0,
			maxValue: 1000,
			maxLength: 3,
			width: 40,
			enableKeyEvents: true,
			msgTarget: 'layoutMarginsMsg'
		};
		this.layoutMarginsVerticalField = new Ext.form.NumberField(
			Ext.apply({
				name: 'verticalMargin',
				listeners: {
					scope: this,
					keyup: this.onLayoutMarginsFieldChange,
					focus: this.onFieldFocus
				}

			}, layoutMarginsConfig)
		);
		this.layoutMarginsHorizontalField = new Ext.form.NumberField(
			Ext.apply({
				name: 'horizontalMargin',
				listeners: {
					scope: this,
					keyup: this.onLayoutMarginsFieldChange,
					focus: this.onFieldFocus
				},
				disabled: this.getDesign().countColumns() == 1
			}, layoutMarginsConfig)
		);

		var layoutMarginsMsg = Ext.apply({id: 'layoutMarginsMsg'}, layoutMsgConfig),
			layoutMarginsColumn = {
				columnWidth: 0.5,
				defaults: {
					border: false
				},
				items: [{
					cls: 'sb-layout-layout-column-title',
					html: Sb.Locale.get('layoutMargins')
				}, {
					layout: 'column',
					defaults: {
						border: false
					},
					items: [{
						cls: 'sb-layout-margins-vertical-column',
						items: [
							layoutMarginsVerticalLabel,
							this.layoutMarginsVerticalField
						]
					}, {
						items: [
							layoutMarginsHorizontalLabel,
							this.layoutMarginsHorizontalField
						]
					}]
				}, layoutMarginsMsg]
			};

		Ext.apply(this, {
			cls: 'sb-editor-layout-panel',
			title: Sb.Locale.get('layoutTitle'),
			helperMessage: Sb.Locale.get('layoutDescription'),
			tabTip: Sb.Locale.get('layoutDescriptionOnHover'),
			labelAlign: 'top',
			autoHeight: true,
			defaults: {
				border: false
			},
			items: [{
				layout: 'column',
				cls: 'sb-layout-containers-controls',
				defaults: {
					border: false
				},
				items: [dollColumn, widthHeightFieldsColumn]
			}, {
				layout: 'column',
				cls: 'sb-layout-layout-controls',
				defaults: {
					border: false
				},
				items: [layoutWidthColumn, layoutMarginsColumn]
			}],
			listeners: {
				scope: this,
				activate: this.onActivate
			}
		});
		Sb.editor.LayoutPanel.superclass.initComponent.apply(this, arguments);
	},

	/**
	 * Activate handler
	 */
	onActivate: function () {
		this.memento = this.getDesign().saveToMemento();
		this.notRegisterChanges = true;

		this.updateDoll();
		this.updateColumnsCountControl();
		this.updateColumnsWidthControl();
		this.updateColumnsHeightControl();
		this.updateColumnsSwitchButton();
		this.updateLayoutWidthControl();
		this.updateLayoutMarginsControl();

		delete this.notRegisterChanges;
	},

	updateDoll: function () {
		var data = this.getLayoutData();

		this.doll.set({
			headerLayout: data.headerLayout,
			contentLayout: data.contentLayout,
			footerLayout: data.footerLayout,
			sidebarFirstVisible: data.sidebarFirst ? true : false,
			sidebarSecondVisible: data.sidebarSecond ? true : false,
			sidebarsSwitched: this.getDesign().isSidebarsSwitched()
		});
	},

	updateColumnsCountControl: function () {
		var countColumns = this.getDesign().countColumns();
		this.layoutSidebarsRadioGroup.setValue(countColumns);
	},

	updateColumnsWidthControl: function () {
		var data = this.getLayoutData(),
			design = this.getDesign();

		this.layoutSidebarLeftWidthColumn.setVisible(design.isLeftColumnVisible());
		this.layoutSidebarRightWidthColumn.setVisible(design.isRightColumnVisible());

		var layoutColumnLeftWidth = design.getSidebarProperty(Sb.Editor.model.isRtl() ? 'right' : 'left', 'Width');
		this.layoutSidebarLeftWidth.setValue(layoutColumnLeftWidth);

		if (data.type == 'fixed') {
			var layoutColumnContentWidth = this.getContentWidth();
			this.layoutSidebarContentWidth.setValue(layoutColumnContentWidth);
		} else {
			this.layoutSidebarContentWidth.setValue('');
		}

		var layoutColumnRightWidth = design.getSidebarProperty(Sb.Editor.model.isRtl() ? 'left' : 'right', 'Width');
		this.layoutSidebarRightWidth.setValue(layoutColumnRightWidth);
	},

	updateColumnsHeightControl: function () {
		var data = this.getLayoutData(),
			design = this.getDesign();

		this.layoutSidebarLeftHeightColumn.setVisible(design.isLeftColumnVisible());
		this.layoutSidebarRightHeightColumn.setVisible(design.isRightColumnVisible());

		var layoutColumnLeftHeight = design.getSidebarProperty(Sb.Editor.model.isRtl() ? 'right' : 'left', 'MinHeight');
		this.layoutSidebarLeftHeight.setValue(layoutColumnLeftHeight);
		this.layoutSidebarContentHeight.setValue(data.contentMinHeight);

		var layoutColumnRightHeight = design.getSidebarProperty(
			Sb.Editor.model.isRtl() ? 'left' : 'right', 'MinHeight'
		);
		this.layoutSidebarRightHeight.setValue(layoutColumnRightHeight);

		this.pressColumnsToBottomCheckbox.setValue(design.isPressColumnsToBottom());
	},

	updateColumnsSwitchButton: function () {
		var design = this.getDesign();
		this.layoutSidebarsSwitchButton.setVisible(design.isLeftColumnVisible() || design.isRightColumnVisible());
	},

	updateLayoutWidthControl: function () {
		var data = this.getLayoutData(),
			width = parseFloat(data.width);

		this.layoutFixedRadio.setValue(data.type == 'fixed');
		this.layoutElasticRadio.setValue(data.type == 'elastic');
		this.layoutWidthField.setMinValue(data.type == 'fixed' ? 100 : 50);
		this.layoutWidthField.setMaxValue(data.type == 'fixed' ? 10000 : 100);
		this.layoutWidthField.setValue(width);
		var unit = data.type == 'elastic' ? '%' : Sb.Locale.get('px');
		this.layoutWidthUnit.update(unit);
		this.layoutWidthField.validate();
	},

	updateLayoutMarginsControl: function () {
		var data = this.getLayoutData();

		this.layoutMarginsVerticalField.setValue(data.verticalMargin);
		this.layoutMarginsHorizontalField.setValue(data.horizontalMargin);
	},

	getDesign: function () {
		return Sb.Editor.model.design;
	},

	getLayoutData: function () {
		return this.getDesign().getSettings().layout;
	},

	setLayoutData: function (data) {
		if (this.notRegisterChanges) {
			return;
		}
		this.getDesign().unserialize({layout: data});
		Sb.editor.UiContext.registerChanges();
	},

	onFieldFocus: function (field) {
		field.validate();
	},

	onDollChange: function (dollLayout) {
		this.setLayoutData(dollLayout);
		this.updateColumnsWidthControl();
	},

	onColumnsCountChange: function (radioGroup, radio) {
		var data = this.getLayoutData(),
			sidebarFirst,
			sidebarSecond,
			value = radio.getGroupValue();

		if (value == this.getDesign().countColumns()) {
			return;
		}

		this.layoutMarginsHorizontalField.setDisabled(value == 1);

		switch (value) {
			case '1':
				sidebarFirst = sidebarSecond = false;
				break;
			case '2':
				var columnsWithContent = this.isContentInSidebar('first') + this.isContentInSidebar('second');

				//switch from 3 => 2 columns
				if (data.sidebarFirst && data.sidebarSecond) {
					if (columnsWithContent == 1) {
						sidebarFirst = this.isContentInSidebar('first');
						sidebarSecond = this.isContentInSidebar('second');
						break;
					}

					var dialog = new Sb.editor.LayoutDialog({
						listeners: {
							scope: this,
							change: this.onDialogChange,
							cancel: this.onDialogCancel
						}
					});
					dialog.show();

					return false;
				}

				//switch from 1 => 2 columns
				if (columnsWithContent == 1) {
					sidebarFirst = this.isContentInSidebar('first');
					sidebarSecond = this.isContentInSidebar('second');
					break;
				}
				sidebarFirst = true;
				break;
			case '3':
				sidebarFirst = sidebarSecond = true;
				break;
		}

		this.setLayoutData({
			sidebarFirst: sidebarFirst,
			sidebarSecond: sidebarSecond
		});

		this.updateDoll();
		this.updateColumnsWidthControl();
		this.updateColumnsHeightControl();
		this.updateColumnsSwitchButton();
	},

	onColumnLeftWidthChange: function (field) {
		var value = field.getValue();

		if (value < 100) {
			return;
		}

		this.setSidebarWidth(value, this.getDesign().isSidebarsSwitched() ? 'Second' : 'First');
	},

	onColumnRightWidthChange: function (field) {
		var value = field.getValue();

		if (value < 100) {
			return;
		}

		this.setSidebarWidth(value, this.getDesign().isSidebarsSwitched() ? 'First' : 'Second');
	},

	onColumnLeftHeightChange: function (field) {
		var value = field.getValue();
		this.setMinHeight(value, this.getDesign().isSidebarsSwitched() ? 'sidebarSecond' : 'sidebarFirst');
	},

	onColumnContentHeightChange: function (field) {
		var value = field.getValue();
		this.setMinHeight(value, 'content');
	},

	onColumnRightHeightChange: function (field) {
		var value = field.getValue();

		this.setMinHeight(value, this.getDesign().isSidebarsSwitched() ? 'sidebarFirst' : 'sidebarSecond');
	},

	onPressColumnsToBottomChange: function (field, checked) {
		this.setLayoutData({pressColumnsToBottom: checked});
	},

	switchColumns: function () {
		var isSidebarsSwitched = this.getDesign().isSidebarsSwitched(),
			sidebarFirstPosition = isSidebarsSwitched ? 'left' : 'right',
			sidebarSecondPosition = isSidebarsSwitched ? 'right' : 'left';

		this.setLayoutData({
			sidebarFirstPosition: sidebarFirstPosition,
			sidebarSecondPosition: sidebarSecondPosition
		});

		this.updateDoll();
		this.updateColumnsWidthControl();
		this.updateColumnsHeightControl();
	},

	onLayoutTypeRadioCheck: function (radio, checked) {
		if (!checked) {
			return;
		}

		var type = radio.getName(),
			data = this.getLayoutData();

		if (data.type == type) {
			return;
		}

		var modelWidth = data.width,
			bodyWidth = Ext.getBody().getWidth();

		var width = (type == 'elastic') ? modelWidth * 100 / bodyWidth
			: Math.round(modelWidth * bodyWidth / 100);

		this.setLayoutData({
			type: type,
			width: width
		});

		this.updateLayoutWidthControl();
		this.updateColumnsWidthControl();
	},

	onLayoutWidthFieldChange: function (field) {
		var width = field.getValue();

		if (field.isValid()) {
			this.setLayoutData({width: width});
			this.updateColumnsWidthControl();
		}
	},

	onLayoutMarginsFieldChange: function (field) {
		var newData = {},
			size = field.getValue(),
			name = field.getName();

		newData[name] = Math.abs(size);
		this.setLayoutData(newData);
		this.updateColumnsWidthControl();

		// :KLUDGE: 2012/04/05 Workaround for bug #102192
		if (name == 'horizontalMargin' && Ext.isChrome) {
			var columnsEl = Ext.get('columns');
			columnsEl.setStyle.defer(100, columnsEl, [{'box-sizing': 'content-box'}]);
			columnsEl.setStyle.defer(110, columnsEl, [{'box-sizing': 'border-box'}]);
		}
	},

	onDialogChange: function (dialogSelection) {
		var data = this.getLayoutData(),
			sidebarFirst;

		sidebarFirst = (dialogSelection == 'hideRight') ?
			data.sidebarFirstPosition == 'left' :
			data.sidebarFirstPosition == 'right';

		if (Sb.Editor.model.isRtl()) {
			sidebarFirst = !sidebarFirst;
		}

		this.setLayoutData({
			sidebarFirst: sidebarFirst,
			sidebarSecond: !sidebarFirst
		});

		this.updateDoll();
		this.updateColumnsCountControl();
		this.updateColumnsWidthControl();
		this.updateColumnsHeightControl();
		this.updateColumnsSwitchButton();
	},

	onDialogCancel: function () {
		this.updateColumnsCountControl();
	},

	/**
	 * Detecting a content in sidebar on all pages
	 * @sidebar {String} sidebar first|second
	 * @return (Boolean)
	 */
	isContentInSidebar: function (sidebar) {
		var isContentSidebar = false;

		sidebar = sidebar == 'first' ? 'column1' : 'column2';
		Sb.Editor.model.rootPage.pages.each(function (page) {
			page.widget.widgets.each(function (widget) {
				if (widget.data.region == sidebar && widget.widgets.getCount()) {
					isContentSidebar = true;
					return false;
				}
			});
			if (isContentSidebar) {
				return false;
			}
		});

		return isContentSidebar;
	},

	getExternalBordersWidth: function (type) {
		var data = this.getLayoutData(),
			externalBordersWidth = Sb.Editor.siteView.pageWidget.getExternalBordersWidth();
		type = type || data.type;

		return (type == 'elastic') ?
			externalBordersWidth * 100 / Ext.getBody().getWidth() :
			externalBordersWidth;
	},

	/**
	 * Get width content area in pixel
	 * @return {Number} Return calculated width for content area
	 */
	getContentWidth: function () {
		var data = this.getLayoutData(),
			firstWidth = data.sidebarFirst ? parseInt(data.sidebarFirstWidth, 10) : 0,
			secondWidth = data.sidebarSecond ? parseInt(data.sidebarSecondWidth, 10) : 0,
			margins = parseInt(data.horizontalMargin, 10) * (this.getDesign().countColumns() - 1),
			contentWidth = parseInt(data.width, 10) - firstWidth - secondWidth - margins;

		return contentWidth > 0 ? Math.round(contentWidth) : 0;
	},

	/**
	 * Set width for sidebar
	 * @param {Number} width
	 * @param {String} name First|Second
	 */
	setSidebarWidth: function (width, name) {
		var newData = {};

		if (Sb.Editor.model.isRtl()) {
			name = Sb.Utils.swapLeftAndRight(name);
		}

		newData['sidebar' + name + 'Width'] = width;
		this.setLayoutData(newData);
	},

	/**
	 * Set minimal height for sidebar
	 * @param {Number} height
	 * @param {String} name sidebarFirst|sidebarSecond|Content
	 */
	setMinHeight: function (height, name) {
		var newData = {};

		if (Sb.Editor.model.isRtl()) {
			name = Sb.Utils.swapLeftAndRight(name);
		}

		height = Math.abs(height);
		if (height < 100) {
			return;
		}
		newData[name + 'MinHeight'] = height;
		this.setLayoutData(newData);
	},

	/**
	 * OK button handler.
	 */
	okHandler: function () {
		var error;

		if (
			!this.layoutSidebarLeftWidth.isValid() ||
			!this.layoutSidebarRightWidth.isValid()
		) {
			error = 'minColumnWidth';
		}

		if (
			!this.layoutSidebarLeftHeight.isValid() ||
			!this.layoutSidebarContentHeight.isValid() ||
			!this.layoutSidebarLeftHeight.isValid()
		) {
			error = 'minColumnHeight';
		}

		if (!this.layoutWidthField.isValid()) {
			error = 'layoutWidthInvalid';
		}

		if (
			!this.layoutMarginsVerticalField.isValid() ||
			!this.layoutMarginsHorizontalField.isValid()
		) {
			error = 'layoutMarginsInvalid';
		}

		if (error) {
			Sb.Editor.Ui.mainWindow.statusBar.setMessageError(Sb.Locale.get(error));
		} else {
			Sb.editor.LayoutPanel.superclass.okHandler.call(this);
		}
	},

	/**
	 * Save handler.
	 * @param {Function} afterSaveCallbacks Handler, that will be run after save
	 */
	saveHandler: function (afterSaveCallbacks) {
		this.getDesign().save({
			success: function () {
				Ext.callback(afterSaveCallbacks.success);
				Sb.Editor.Ui.mainWindow.statusBar.setMessage(
					Sb.Locale.get('messageChangesSaved')
				);
			},
			failure: afterSaveCallbacks.failure
		});
	},

	/**
	 * Discard handler
	 */
	discardHandler: function () {
		this.getDesign().restoreFromMemento(this.memento);
	}
});