/**
 * Edit dialog for model
 *
 * @class Sb.editor.ModelEditDialog
 * @extends Sb.Window
 */
Ext.define('Sb.editor.ModelEditDialog', {
	extend: 'Sb.Window',

	modal: true,
	resizable: false,
	closeAction: 'cancelHandler',
	width: 475,

	constructor: function (config) {
		config = Ext.apply({
			/**
			 * Model for editing
			 *
			 * @type {Sb.model.Node}
			 * @property model
			 */
			model: null,
			/**
			 * Data for model
			 * @type {Object}
			 */
			modelData: {},
			/**
			 * Default class for model
			 * @type {String}
			 */
			modelClass: Sb.model.Node,
			/**
			 * Memento for reverting on Cancel
			 *
			 * @type {Object}
			 * @property memento
			 */
			memento: null
		}, config);

		Sb.editor.ModelEditDialog.superclass.constructor.call(this, config);
	},

	initComponent: function () {
		this.buttons = [{
			text: Sb.Locale.get('ok'),
			handler: this.okHandler.createDelegate(this)
		}, {
			text: Sb.Locale.get('cancelButtonTitle'),
			handler: this.cancelHandler.createDelegate(this)
		}];
		Sb.editor.ModelEditDialog.superclass.initComponent.call(this);

		this.initModel(this.modelData);
	},

	/**
	 * Create model and save to memento
	 */
	initModel: function (config) {
		config = config || {};
		if (null == this.model) {
			this.model = new this.modelClass(config);
		}
		this.memento = this.model.saveToMemento();
	},

	/**
	 * Init events
	 */
	initEvents: function () {
		Sb.editor.ModelEditDialog.superclass.initEvents.call(this);
		this.model.on('load', this.fillDialogFromModel, this);
	},

	/**
	 * Fill dialog from model
	 */
	fillDialogFromModel: Ext.emptyFn,

	/**
	 * Clean up events
	 */
	onDestroy: function () {
		this.model.un('load', this.fillDialogFromModel, this);
		this.model.destroy();
	},

	/**
	 * OK button handler
	 */
	okHandler: function () {
		this.destroy();
	},

	/**
	 * Cancel button handler
	 */
	cancelHandler: function () {
		this.model.restoreFromMemento(this.memento);
		this.destroy();
	}

});