/**
 * @class Sb.editor.StyleBorderPanel
 * @extends Ext.Panel
 * Style border editor panel
 * @constructor
 * @param {Object} config The config object
 * @xtype styleborderpanel
 */
Ext.define('Sb.editor.StyleBorderPanel', {
	extend: 'Ext.Panel',
	xtype: 'styleborderpanel',

	// private
	initComponent: function () {
		var siteModel = Sb.Editor.model;
		var borderTab = {
			xtype: 'container',
			defaults: {border: false},
			items: [{
				layout: 'column',
				defaults: {border: false},
				cls: 'sb-style-borders-innerborders',
				items: [{
					width: 244,
					layout: 'form',
					cls: 'sb-style-border-typeselection',
					items: {
						xtype: 'sbselectbox',
						ref: '../../../borderList',
						cls: 'sb-style-border',
						itemClass: 'sb-style-border-list-item',
						triggerClass: 'sb-style-border-trigger',
						fieldLabel: Sb.Locale.get('borderContainersWithinSite'),
						triggerAction: 'all',
						store: new Ext.data.ArrayStore({
							fields: ['name']
						}),
						mode: 'local',
						valueField: 'name',
						width: 228,
						listWidth: 442,
						lazyInit: false,
						itemTpl: '<div style="background-image: url(' + Sb.Config.templatesUrl + '/' +
							siteModel.data.template.code + '/decorations/border/{name}/images/{name}.png);"></div>',
						itemAspectRatio: 3.8,
						columns: 2,
						rows: 5,
						lastQuery: '',
						onActivate: function () {
							var innerBorder = 'border-none';
							Ext.iterate(siteModel.design.data.border, function (region, border) {
								if (border != innerBorder) {
									innerBorder = border;
									return false;
								}
							});
							if (innerBorder == 'border-none') {
								innerBorder = 'border1-black';
							}
							this.setValue(innerBorder);

							var matches = innerBorder.match(/(black|white)$/);
							Ext.each(this.header.findByType('radio'), function (radio) {
								radio.setValue(false);
								if (radio.value == (matches ? matches[1] : 'black')) {
									radio.setValue(true);
								}
							});
						},
						header: {
							xtype: 'container',
							cls: 'sb-invert-colors',
							items: [{
								xtype: 'label',
								html: Sb.Locale.get('invertBorderColors'),
								cls: 'sb-invert-colors-title'
							}, {
								xtype: 'radio',
								name: 'invertBorderColors',
								boxLabel: Sb.Locale.get('black'),
								value: 'black',
								listeners: {
									scope: this,
									check: this.onChangeBorderColor
								}
							}, {
								xtype: 'radio',
								name: 'invertBorderColors',
								boxLabel: Sb.Locale.get('white'),
								value: 'white',
								listeners: {
									scope: this,
									check: this.onChangeBorderColor
								}
							}]
						},
						listeners: {
							scope: this,
							select: function (selectBox, record, index) {
								this.findByType('checkboxgroup')[0].eachItem(function (checkBox) {
									if (checkBox.getValue()) {
										siteModel.design.setDecoration('border', checkBox.region + 'Border', record.get('name'));
									}
								});
								Sb.editor.UiContext.registerChanges();
							},
							afterrender: function (selectBox) {
								var data = [];
								Ext.iterate(Sb.Editor.model.data.template.decorations.borders, function (key) {
									data.push([key]);
								});
								selectBox.store.loadData(data);
							}
						}
					}
				}, {
					columnWidth: 1,
					layout: 'form',
					cls: 'sb-style-border-locationselection11',
					items: {
						xtype: 'checkboxgroup',
						columns: 2,
						fieldLabel: Sb.Locale.get('StyleBorderDisplayBorderHere'),
						anchor: '100%',
						defaults: {
							listeners: {
								scope: this,
								check: this.onBorderToContainer
							}
						},
						onActivate: function () {
							this.eachItem(function (item) {
								if (item.onActivate) {
									item.onActivate();
								}
							});
						},
						items: [{
							boxLabel: Sb.Locale.get('layoutBorderSidebarLeft'),
							region: 'column1',
							onActivate: function () {
								var sidebarFirstPosition = siteModel.isRtl()
									? Sb.Utils.swapLeftAndRight(siteModel.design.data.layout.sidebarFirstPosition)
									: siteModel.design.data.layout.sidebarFirstPosition;
								this.region = sidebarFirstPosition == 'left' ? 'column1' : 'column2';
								this.setValue(siteModel.design.data.border[this.region + 'Border'] != 'border-none');
								var sidebar = this.region == 'column1' ? 'sidebarFirst' : 'sidebarSecond';
								this.el.up('.x-form-item').dom.style.display = siteModel.design.data.layout[sidebar] ? 'block' : 'none';
							}
						}, {
							boxLabel: Sb.Locale.get('layoutBorderHeader'),
							region: 'header',
							onActivate: function () {
								this.setValue(siteModel.design.data.border.headerBorder != 'border-none');
							}
						}, {
							boxLabel: Sb.Locale.get('layoutBorderColumn2Content'),
							region: 'content',
							onActivate: function () {
								this.setValue(siteModel.design.data.border.contentBorder != 'border-none');
							}
						}, {
							boxLabel: Sb.Locale.get('layoutBorderFooter'),
							region: 'footer',
							onActivate: function () {
								this.setValue(siteModel.design.data.border.footerBorder != 'border-none');
							}
						}, {
							boxLabel: Sb.Locale.get('layoutBorderSidebarRight'),
							region: 'column2',
							onActivate: function () {
								var sidebarFirstPosition = siteModel.isRtl()
									? Sb.Utils.swapLeftAndRight(siteModel.design.data.layout.sidebarFirstPosition)
									: siteModel.design.data.layout.sidebarFirstPosition;
								this.region = sidebarFirstPosition == 'right' ? 'column1' : 'column2';
								this.setValue(siteModel.design.data.border[this.region + 'Border'] != 'border-none');
								var sidebar = this.region == 'column1' ? 'sidebarFirst' : 'sidebarSecond';
								this.el.up('.x-form-item').dom.style.display = siteModel.design.data.layout[sidebar] ? 'block' : 'none';
							}
						}]
					}
				}]
			}, {
				xtype: 'container',
				layout: 'form',
				cls: 'sb-style-borders-siteborder',
				items: {
					xtype: 'sbselectbox',
					ref: '../../externalBorderList',
					cls: 'sb-style-border',
					itemClass: 'sb-style-border-list-item',
					triggerClass: 'sb-style-border-trigger',
					fieldLabel: Sb.Locale.get('borderAroundSite'),
					triggerAction: 'all',
					store: new Ext.data.ArrayStore({
						fields: ['name']
					}),
					mode: 'local',
					width: 228,
					valueField: 'name',
					listWidth: 442,
					lazyInit: false,
					itemTpl: '<div style="background-image: url(' + Sb.Config.templatesUrl + '/' +
						siteModel.data.template.code + '/decorations/external-border/{name}/images/{name}.png);">' +
						'</div>',
					itemAspectRatio: 3.8,
					columns: 2,
					rows: 5,
					lastQuery: '',
					onActivate: function () {
						var border = siteModel.design.data['external-border'].watermarkExternalBorder;
						this.setValue(border);

						var matches = border.match(/(black|white)$/);
						Ext.each(this.header.findByType('radio'), function (radio) {
							radio.setValue(false);
							if (radio.value == (matches ? matches[1] : 'black')) {
								radio.setValue(true);
							}
						});
					},
					header: {
						xtype: 'container',
						cls: 'sb-invert-colors',
						items: [{
							xtype: 'label',
							html: Sb.Locale.get('invertBorderColors'),
							cls: 'sb-invert-colors-title'
						}, {
							xtype: 'radio',
							name: 'invertExternalBorderColors',
							boxLabel: Sb.Locale.get('black'),
							value: 'black',
							listeners: {
								scope: this,
								check: this.onChangeBorderColor
							}
						}, {
							xtype: 'radio',
							name: 'invertExternalBorderColors',
							boxLabel: Sb.Locale.get('white'),
							value: 'white',
							listeners: {
								scope: this,
								check: this.onChangeBorderColor
							}
						}]
					},
					listeners: {
						scope: this,
						select: function (selectBox, record, index) {
							siteModel.design.setDecoration('external-border', 'watermarkExternalBorder', record.get('name'));
							Sb.editor.UiContext.registerChanges();
						},
						afterrender: function (selectBox) {
							var data = [];
							Ext.iterate(Sb.Editor.model.data.template.decorations.externalBorders, function (key) {
								data.push([key]);
							});
							selectBox.store.loadData(data);
						}
					}
				}
			}]
		};

		Ext.apply(this, {
			title: Sb.Locale.get('styleBordersTab'),
			cls: 'sb-editor-style-border-panel',
			defaults: {
				border: false,
				layout: 'form',
				labelAlign: 'top'
			},
			helperMessage: Sb.Locale.get('styleBorderTabDescription'),
			items: borderTab,
			listeners: {
				scope: this,
				activate: this.onActivate
			}
		});
		Sb.editor.StyleBorderPanel.superclass.initComponent.apply(this, arguments);
	},

	/**
	 * Change border to selected color
	 * @param {Ext.form.Radio} radio
	 * @param {Sb.form.SelectBox} selectBox
	 */
	onChangeBorderColor: function (radio) {
		if (!radio.checked) {
			return;
		}
		var subPattern = radio.value + (radio.name == 'invertExternalBorderColors' ? '|none' : '');
		var pattern = new RegExp('(.+)-(' + subPattern + ')', 'i');
		var store = (radio.name == 'invertExternalBorderColors' ? this.externalBorderList : this.borderList).store;

		store.filterBy(function (record) {
			return pattern.test(record.get('name'));
		});
	},

	/**
	 * Activate values and states of elements for border
	 */
	onActivate: function () {
		this.borderList.onActivate();
		this.externalBorderList.onActivate();
		Ext.each(['checkboxgroup'], function (xtype) {
			Ext.each(this.findByType(xtype), function (item){
				if (item.onActivate) {
					item.onActivate();
				}
			});
		}, this);
	},

	/**
	 * Set Unset border to some container
	 * @param {Ext.form.Checkbox} checkbox
	 * @param {Boolean} checked
	 */
	onBorderToContainer: function (checkbox, checked) {
		var selectedBorder = checked ? this.borderList.getValue() : 'border-none';
		Sb.Editor.model.design.setDecoration('border', checkbox.region + 'Border', selectedBorder);
		Sb.editor.UiContext.registerChanges();
	}
});