/**
 * Class for image component edit panel
 *
 * @class Sb.editor.component.DateEditPanel
 * @extends Sb.editor.component.BaseEditPanel
 */
Ext.define('Sb.editor.component.DateEditPanel', {
	extend: 'Sb.editor.component.BaseEditPanel',

	cls: 'sb-component-edit-panel-date',
	autoHeight: true,

	initComponent: function () {
		this.picker = this.items = new Ext.DatePicker({
			showToday: false,
			focusOnSelect: false,
			weekendDays: [0, 6],
			listeners: {
				scope: this,
				select: function (picker, date) {
					this.component.setValue(date);
					this.component.completeEdit(true);
				}
			},
			// owerride default method for custom view
			update: function (date, forceRefresh) {
				var vd = this.activeDate;
				Ext.DatePicker.prototype.update.apply(this, [date, forceRefresh]);

				if (!this.rendered || !forceRefresh && vd
					&& vd.getMonth() == date.getMonth()
					&& vd.getFullYear() == date.getFullYear()
				) {
					return;
				}

				var i, day, cells, header, newHeader, monthStore, yearStore,
					wdays = this.weekendDays ? this.weekendDays.join('') : false;

				if (!this.hacked) {
					cells = this.el.select('table.x-date-inner thead th');
					for (i = 0; i < 7; i++) {
						if(wdays.indexOf((i + this.startDay) % 7) != -1) {
							cells.elements[i].className += ' x-date-weekend';
						}
					}

					header = this.el.down('table tr', true);
					header.style.display = 'none';
					newHeader = Ext.DomHelper.insertHtml('afterEnd', header, '<tr><td colspan="3"></td></tr>');

					monthStore = [];
					for (i = 0; i < 12; i++) {
						monthStore.push([i, this.monthNames[i]]);
					}
					this.monthCombo = new Ext.form.ComboBox({
						renderTo: newHeader.firstChild,
						cls: 'sb-component-edit-panel-date-month-combo',
						store: monthStore,
						listeners: {
							scope: this,
							select: function (combo, record) {
								var d = new Date(this.yearCombo.getValue(), this.monthCombo.getValue(), (this.activeDate || this.value).getDate());
								this.update(d);
							}
						}
					});

					yearStore = [];
					for (i = 1970; i < 2039; i++) {
						yearStore.push(i);
					}
					this.yearCombo = new Ext.form.ComboBox({
						renderTo: newHeader.firstChild,
						cls: 'sb-component-edit-panel-date-year-combo',
						store: yearStore,
						listeners: {
							scope: this,
							select: function (combo, record) {
								var d = new Date(this.yearCombo.getValue(), this.monthCombo.getValue(), (this.activeDate || this.value).getDate());
								this.update(d);
							}
						}
					});

					this.hacked = true;
				}

				var value = this.value.getTime();
				for (i = 0; i < 42; i++) {
					var cell = this.cells.elements[i];
					if (cell.firstChild.dateValue == value) {
						cell.className += ' x-date-value';
					}
					if (wdays && wdays.indexOf((i + this.startDay) % 7) != -1) {
						cell.className += ' x-date-weekend';
					}
				}

				this.monthCombo.setValue(date.getMonth());
				this.yearCombo.setValue(date.getFullYear());
			}
		});

		Sb.editor.component.DateEditPanel.superclass.initComponent.apply(this, arguments);
	},

	show: function () {
		Sb.editor.component.DateEditPanel.superclass.show.apply(this, arguments);
		this.picker.activeDate = null;
		this.picker.setValue(this.component.getValue());
	}
});