/**
 * @class Sb.form.ContainerComboBox
 * @extends Ext.form.ComboBox
 * Class for container select
 * @constructor
 * @param {Object} config The config object
 * @xtype containercombobox
 */
Ext.define('Sb.form.ContainerComboBox', {
	extend: 'Ext.form.ComboBox',
	xtype: 'containercombobox',

	constructor: function (config) {
		config = Ext.apply({
			mode: 'local',
			fieldLabel: Sb.Locale.get('containerComboBoxLabel'),
			editable: false,
			triggerAction: 'all',
			store: new Ext.data.ArrayStore({
				fields: ['value', 'name']
			}),
			displayField: 'name',
			valueField: 'value',
			autoSelect: true,
			lazyInit: false,
			pageAreas: ['watermark', 'header', 'footer', 'content']
		}, config);

		Sb.form.ContainerComboBox.superclass.constructor.call(this, config);
	},

	initEvents: function () {
		Sb.form.ContainerComboBox.superclass.initEvents.apply(this, arguments);
		this.mon(Sb.Editor.model.design, 'layoutchange', this.updatePlacesData, this);
	},

	// private
	afterRender: function () {
		Sb.form.ContainerComboBox.superclass.afterRender.call(this);
		this.updatePlacesData();
	},

	updatePlacesData: function () {
		var placesData = [], keyName,
			layout = Sb.Editor.model.design.getSettings().layout;
		Ext.iterate(this.pageAreas, function (place) {
			placesData.push([place, Sb.Locale.get(place + 'Title')]);
		});
		if (layout.sidebarFirst) {
			keyName = (layout.sidebarFirstPosition == 'left' ? 'column1' : 'column2') + 'Title';
			placesData.push(['column1', Sb.Locale.get(keyName)]);
		}
		if (layout.sidebarSecond) {
			keyName = (layout.sidebarSecondPosition == 'left' ? 'column1' : 'column2') + 'Title';
			placesData.push(['column2', Sb.Locale.get(keyName)]);
		}
		this.store.loadData(placesData);
		this.setValue('content');
	}
});