/**
 * @class Sb.form.FieldGroup
 * @extends Ext.Container
 * @xtype fieldgroup
 */
Ext.define('Sb.form.FieldGroup', {
	extend: 'Ext.Container',
	xtype: 'fieldgroup',

	cls: 'sb-field-group',

	constructor: function (config) {
		config = config || {};
		config = Ext.apply({
			layout			: 'form',
			border			: false,
			autoWidth		: (!config.width && !config.columnWidth) ? true : false,
			labelAlign		: 'top'
		}, config);
		if ('card' == config.layout) {
			config = Ext.apply({
				layoutConfig : {
					deferredRender : true
				}
			}, config);
		}
		Sb.form.FieldGroup.superclass.constructor.call(this, config);
	},

	/**
	 * Tries to find component and toggle it by show/hide.
	 * If state is defined, then tries to set it`s visibility in that state.
	 *
	 * @param {String|Ext.Component} component itemId of component or just component
	 * @param {Boolean} state State, that default undefined
	 */
	toggle: function (component, state) {
		if (Ext.isString(component)) {
			var componentId = component;
			component = this.getComponent(componentId);
			if (undefined === component) {
				throw new Error('Component with itemId ' + componentId + ' not found');
			}
		}
		if (undefined === state) {
			state = component.hidden;
		}
		var fn = this.doToggle.createDelegate(this, [component, state]);
		if (!component.rendered) {
			component.on('render', fn, null, {single: true});
		} else {
			fn();
		}
	},

	/**
	 * Toggle component by show or hide as in state.
	 *
	 * @param {Ext.Component} component
	 * @param {Boolean} state
	 */
	doToggle: function (component, state) {
		var toggleObj		= component;
		if (component.itemCt && component.itemCt != this) {
			toggleObj		= component.itemCt;
			toggleObj.setVisibilityMode(Ext.Element.DISPLAY);
		}
		toggleObj.setVisible(state);
	},

	/**
	 * Adds callback on actual component show, and once after it rendered
	 * @param callback
	 * @param scope
	 * @param showCaller
	 */
	onActualShow: function (callback, scope, showCaller) {
		showCaller	= showCaller || this;
		scope		= scope || this;
		this.on('afterlayout', function () {
			Ext.callback(callback, scope);
			showCaller.on('show', callback, scope);
		}, this, {single: true});
	}
});