/**
 * @class Sb.form.FileComboBox
 * @extends Ext.form.ComboBox
 */
Ext.define('Sb.form.FileComboBox', {
	extend: 'Ext.form.ComboBox',
	xtype: 'sbfilecombobox',

	placeholderId: 0,
	msgTarget: 'under',

	initComponent: function () {
		this.placeholderId = Ext.id();
		this.focusLinkId = Ext.id();

		Ext.apply(this, {
			store: new Ext.data.SimpleStore({fields: [], data: [[]]}),
			editable: false,
			shadow: false,
			mode: 'local',
			triggerAction: 'all',
			tpl: '<tpl for="."><div><div id="'+this.placeholderId+'"></div><a href="#" class="x-grid3-focus" id="'+this.focusLinkId+'" tabIndex="-1"/></div></tpl>',
			selectedClass: '',
			onSelect: Ext.emptyFn,
			displayField: 'name',
			valueField: 'file'
		});

		this.documentManager = new Sb.editor.DocumentManager({
			model: Sb.Editor.documents,
			height: 252,
			listeners: {
				scope: this,
				selectionchange: function (dataView) {
					var records = dataView.getSelectedRecords();
					if (records.length == 1) {
						var record = records[0];
						this.setValue(record.get('file'));
						this.collapse();
					}
				}
			}
		});

		this.documentManagerPanel = new Ext.form.FormPanel({
			border: false,
			fileUpload: true,
			items: [
				this.documentManager,
				{
					xtype: 'container',
					cls: 'sb-file-combobox-footer',
					items: [{
						xtype		: 'fileuploadfield',
						buttonOnly	: true,
						multiple	: true,
						name		: 'document',
						msgTarget	: 'under',
						buttonCfg	: {
							text	: Sb.Locale.get('upload'),
							width	: 100
						},
						listeners: {
							scope: this,
							render: function (cmp) {
								cmp.button.setIconClass('sb-upload-icon');
							},
							click: function (e) { e.stopPropagation(); },
							fileselected: this.onUpload
						}
					}]
				}
			],
			listeners: {
				scope: this,
				afterrender: function () {
					if (this.documentManager.model.isLoaded) {
						this.documentManager.refresh();
					} else {
						this.documentManager.loadDocuments();
					}
				}
			}
		});

		this.on('expand', this.onExpand);

		Sb.form.FileComboBox.superclass.initComponent.call(this);
	},

	setValue: function (v) {
		var text = this.valueField ? this.documentManager.model.getNameByUrl(v) : v;
		this.lastSelectionText = text;
		if (this.hiddenField) {
			this.hiddenField.value = Ext.value(v, '');
		}
		Ext.form.ComboBox.superclass.setValue.call(this, text);
		this.value = v;
		return this;
	},

	assertValue: function () {
		var val = this.getRawValue();
		if (this.valueField) {
			if (this.value == val) {
				return;
			}
			val = this.documentManager.model.getUrlByName(val);
		}
		this.setValue(val);
	},

	onFocus: function () {
		Sb.form.FileComboBox.superclass.onFocus.call(this);
		Ext.get(this.focusLinkId).focus();
	},

	onExpand: function () {
		if (this.documentManagerPanel.rendered) {
			this.documentManager.refresh();
		} else {
			this.documentManagerPanel.render(this.placeholderId);
		}
		this.documentManager.focus();
	},

	onViewClick : function (doFocus) {
		//do nothing
	},

	onUpload: function (field, fieldValue) {
		if ('' === fieldValue) {
			return;
		}

		var form = this.documentManagerPanel.getForm(),
			url = 'Upload/Document',
			params = {'X-CSRF-Token': Sb.Config.csrfToken},
			waitMsg = Sb.Locale.get('uploadingDocument');

		if (Ext.isIE) {
			form.submit({
				url: url,
				params: params,
				scope: this,
				clientValidation: false,
				waitMsg: waitMsg,
				success: function (fp, action) {
					this.afterUploadFiles(action.result);
					field.reset();
				},
				failure: function (fp, action) {
					this.markInvalid(Sb.Locale.get('uploadingError'));
					this.collapse();
					field.reset();
				}
			});
		} else {
			var result = {
					errors: [],
					files: []
				},
				uploader = new Ext.ux.uploader.Manager({
					url: url,
					params: params,
					waitMsg: waitMsg,
					items: field.getFiles(),
					listeners: {
						scope: this,
						itemuploadfailure: function (item, info) {
							if (info.result.errors) {
								result.errors = result.errors.concat(info.result.errors);
							} else {
								result.errors.push(Sb.Locale.get('uploadingError'));
							}
						},
						itemuploadsuccess: function (item, info) {
							result.files = result.files.concat(info.result.files);
						},
						completeupload: function () {
							this.afterUploadFiles(result);
							field.reset();
						}
					}
				});
			uploader.startUpload();
		}
	},

	afterUploadFiles: function (result) {
		this.documentManager.addDocuments(result.files);
		if (result.errors.length > 0) {
			this.markInvalid(result.errors.join('<br />'));
			this.collapse();
		} else {
			if (result.files.length == 1) {
				this.setValue(result.files[0].file);
				this.collapse();
			}
		}
	},

	getListParent: function () {
		return this.el.up('.x-panel') || document.body;
	},

	onDestroy: function () {
		this.documentManagerPanel.destroy();
		Sb.form.FileComboBox.superclass.onDestroy.call(this);
	}
});