/**
 * @class Sb.form.HeaderSelectBox
 * @extends Sb.form.SelectBox
 */
Ext.define('Sb.form.HeaderSelectBox', {
	extend: 'Sb.form.SelectBox',
	xtype: 'sbheaderselectbox',

	filter: null,

	initComponent: function () {
		this.addEvents('deleteheader');

		Ext.applyIf(this, {
			validExtensions: Sb.Config.headerValidExtensions,
			imageUploadErrorMessage: Sb.Locale.get('headerUploadErrorTypeInvalid')
		});

		Ext.apply(this, {
			fieldLabel: Sb.WidgetTypeManager.get('header').getTranslation('imageListTitle'),
			labelSeparator: '',
			triggerAction: 'all',
			mode: 'local',
			width: 238,
			valueField: 'path',
			cls: 'sb-selectbox sb-header-images',
			triggerClass: 'sb-header-images-trigger',
			store: Sb.HeaderManager.headers,
			lazyInit: false,
			listEmptyText: Sb.WidgetTypeManager.get('header').getTranslation('imageListNoMatch'),
			itemRenderer: function (item) {
				if (Sb.utils.Flash.isFlash(item.path)) {
					return Sb.utils.Flash.createCode(item.path, item.width, item.height);
				}
				if (item.path === Sb.Config.headersUrl + '/fallback/preview.png') {
					return '<span class="sb-no-header-title">' +
						Sb.WidgetTypeManager.get('header').getTranslation('imageListNoHeaderTitle') +
						'</span>';
				}
				return '<div style="background: url(' + Sb.Utils.antiCache(item.path) + ')' +
					'no-repeat center;background-size: contain;height: 100%;"></div>';
			},
			itemClass: 'sb-selectbox-list-item sb-header-images-list-item',
			itemAspectRatio: 1.33,
			columns: 5,
			rows: 4,
			itemTools: {
				'delete': {
					scope: this,
					isAvailable: function (itemData) {
						return itemData.isCustom;
					},
					handler: function (selectbox, record) {
						var imageName = record.get('name'),
							imagePath = Sb.model.HeaderWidget.buildImagePath(imageName),
							isUsedImage = null !== Sb.WidgetTypeManager.get('header').cache.find(function (header) {
								return header.get('imagePath') === imagePath &&
									(Sb.Editor.Ui.editComponent ? header !== Sb.Editor.Ui.editComponent.model : true);
							}, this),
							design = Sb.Editor.model.design,
							siteBackground = design.getBackground('site'),
							fallback, index;

						isUsedImage = isUsedImage || siteBackground.image.indexOf('Header/' + imageName) !== -1;

						selectbox.dialog = Ext.Msg.show({
							title: Sb.WidgetTypeManager.get('header').getTranslation('dialogTitleDeleteImage'),
							msg: Sb.WidgetTypeManager.get('header').getTranslation(
								isUsedImage ? 'dialogMsgConfirmDeleteUsingImage' : 'dialogMsgConfirmDeleteImage'
							),
							closable: false,
							buttons: Ext.Msg.YESNO,
							scope: this,
							fn: function (btn) {
								if (btn == 'yes') {
									this.store.remove(record);

									index = this.store.indexOfId('system.fallback');
									fallback = this.store.getAt(index !== -1 ? index : 0);

									if (record.get('path') === this.getValue()) {
										this.selectByValue(fallback.get('path'));
										this.setValue(fallback.get('path'));
									}

									this.fireEvent('deleteheader', record);
								}
							}
						});
					}
				}
			},
			header: [{
				xtype: 'container',
				cls: 'sb-library-panel',
				items: [{
					xtype: 'form',
					fileUpload: true,
					border: false,
					hideLabels: true,
					cls: 'sb-selectbox-list-upload',
					items: {
						xtype: 'fileuploadfield',
						buttonOnly: true,
						buttonText: Sb.WidgetTypeManager.get('header').getTranslation('uploadButton'),
						regex: new RegExp('\\.(' + this.validExtensions.join('|') + ')$', 'i'),
						regexText: this.imageUploadErrorMessage,
						listeners: {
							scope: this,
							fileselected: this.onFileSelected
						}
					}
				}, {
					cls: 'sb-library-panel-separator',
					xtype: 'box'
				}, {
					xtype: 'button',
					text: Sb.Locale.get('search'),
					cls: 'sb-selectbox-list-search-button',
					listeners	: {
						scope: this,
						click: function (button) {
							this.filterHeadersByWord(button.ownerCt.ownerCt.searchField.getValue());
						}
					}
				}, {
					xtype: 'textfield',
					ref: '../searchField',
					emptyText: Sb.WidgetTypeManager.get('header').getTranslation('imageListEmptySearch'),
					validateOnBlur: false,
					cls: 'sb-selectbox-list-search',
					listeners: {
						scope: this,
						valid: function (field) {
							this.filterHeadersByWord(field.getValue());
						}
					}
				}]
			}]
		});
		this.callParent();
	},

	expand: function () {
		this.store.clearFilter();
		if (this.filter) {
			this.store.filterBy(this.filter);
		}

		this.callParent();
	},

	filterHeadersByWord: function (word) {
		var words, wordsRe;

		if (word === '') {
			this.store.clearFilter();
			if (this.filter) {
				this.store.filterBy(this.filter);
			}

			return;
		}

		words = word.trim()
			.split(/[~!@#%\^&:%_, <>\-\/\u3001]/)
			.filter(function (item) { return !Ext.isEmpty(item); })
			.map(Ext.escapeRe)
			.join('|');

		wordsRe = new RegExp('(^|,|\u3001)(' + words + ')', 'ig');

		Sb.HeaderManager.headers.filterBy(function (record) {
			return !record.data.isCustom
				&& record.data.keywords.match(wordsRe);
		});
	},

	onFileSelected: function (field) {
		if (!field.isValid()) {
			Ext.Msg.show({
				title: Sb.Locale.get('imageUploadErrorDialogTitle'),
				msg: field.getActiveError(),
				buttons: Ext.Msg.OK
			});
			field.resetOnUpload();
			return;
		}

		field.ownerCt.getForm().submit({
			url: Sb.Config.baseUrl + '/Upload/Header/',
			waitMsg: Sb.Locale.get('uploadingImage'),
			scope: this,
			success: function (fp, action) {
				this.afterUpload(action.result);
				field.resetOnUpload();
			},
			failure: function (fp, action) {
				if (action.result.message) {
					Ext.Msg.show({
						title: Sb.Locale.get('imageUploadErrorDialogTitle'),
						msg: Sb.Locale.get(action.result.message),
						buttons: Ext.Msg.OK
					});
				}
				field.resetOnUpload();
			}
		});
	},

	afterUpload: function (uploadResult) {
		var name, path, record;

		this.collapse();

		name = Sb.utils.Flash.isFlash(uploadResult.uploadedFileName)
			? uploadResult.uploadedFileName
			: 'thumbnail_' + uploadResult.uploadedFileName;

		path = Sb.Editor.model.getUrl() + '/' + Sb.model.HeaderWidget.buildImagePath(name) + '?' + uploadResult.time;

		record = new this.store.recordType({
			isCustom: true,
			name: uploadResult.uploadedFileName,
			path: path,
			flashWidth: uploadResult.flashWidth,
			flashHeight: uploadResult.flashHeight,
			time: uploadResult.time
		});

		this.header.searchField.setRawValue('');
		this.store.clearFilter();
		// Always insert after "No image"
		this.store.insert(1, record);
		this.store.applySort();
		this.selectByValue(path);
		this.onSelect(record, this.selectedIndex);
	}
});