/**
 * @class Sb.form.PageComboBox
 * @extends Sb.form.TreeComboBox
 */
Ext.define('Sb.form.PageComboBox', {
	extend: 'Sb.form.TreeComboBox',
	xtype: 'sbpagecombobox',

	initComponent: function () {
		var treeNodeUiProvider = Ext.define(null, {
			extend: 'Sb.editor.MoveableTreeNodeUI',
			controls: ''
		});

		this.treeConfig = {
			cls: 'sb-pages-tree',
			lines: false,
			loader: new Ext.tree.TreeLoader({
				doPreload: function (node) {
					node.beginUpdate();
					node.attributes.model.pages.each(function (page) {
						var masterWidget = page.getMasterWidget(),
							pageType = masterWidget ? masterWidget.get('type').camelize().toLowerCase() : 'text';

						if (masterWidget instanceof Sb.model.BlogPostWidget ||
							page.get('sitePageTypeId') == Sb.model.SitePage.LINK_TYPE
						) {
							return true;
						}

						var data = page.data,
							cls = [];
						if (data.hidden) {
							cls.push('sb-pages-tree-node-hidden');
						}
						if (page.isProtected()) {
							cls.push('sb-pages-tree-node-protected');
						}
						var attr = {
								expanded	: true,
								id			: data.identity,
								text		: data.title,
								iconCls		: 'sb-' + pageType + '-icon',
								cls			: cls.join(' '),
								model		: page,
								maxDepth	: Sb.Config.pageMaxLevel,
								uiProvider	: treeNodeUiProvider
							},
							isEmpty	= page.pages.getCount() === 0,
							newNode = isEmpty ? new Ext.tree.TreeNode(attr) : new Ext.tree.AsyncTreeNode(attr);

						node.appendChild(newNode);
						this.doPreload(newNode);
					}, this);

					Ext.each(node.attributes.model.anchors, function (anchor, index) {
						var attr = {
								expanded	: false,
								id			: node.attributes.id + '#' + anchor.name,
								text		: anchor.name,
								iconCls		: 'sb-anchor-icon',
								uiProvider	: treeNodeUiProvider
							},
							newNode = new Ext.tree.TreeNode(attr);
						node.appendChild(newNode);
					});

					node.endUpdate();
					return true;
				}
			}),
			rootVisible: false,
			root: {
				expanded: true,
				leaf: true
			}
		};

		Sb.form.PageComboBox.superclass.initComponent.call(this);

		this.tree.loader.doPreload(this.tree.getRootNode().appendChild(new Ext.tree.AsyncTreeNode({
			expanded: true,
			id: Sb.Editor.model.rootPage.data.identity,
			text: Sb.Locale.get('rootPageTitle'),
			iconCls: 'sb-home-icon',
			model: Sb.Editor.model.rootPage,
			uiProvider: treeNodeUiProvider
		})));
	}
});