/**
 * @class Sb.model.SliderWidget
 * @extends Sb.model.Widget
 * Class for slider model
 * @constructor
 * @param {Object} config The configuration options for the widget model
 */
Ext.define('Sb.model.SliderWidget', {
	extend: 'Sb.model.Widget',

	constructor: function (config) {
		config = Ext.apply({
			type: 'slider'
		}, config);

		/**
		 * The store of images
		 * @type Ext.data.JsonStore
		 * @property images
		 */
		this.images = new Ext.data.JsonStore({
			fields: ['image', 'description', 'link', {
				name: 'translations',
				defaultValue: {}
			}],
			listeners: {
				scope: this,
				add: function () {
					this.updateIsConfiguredProperties();
					this.fireEvent('change', this);
				},
				remove: function () {
					this.updateIsConfiguredProperties();
					this.fireEvent('change', this);
				}
			}
		});

		Sb.model.SliderWidget.superclass.constructor.call(this, config);
	},

	changeLocale: function (newLocale) {
		var oldLocale = this.localeCode || Sb.Editor.model.get('locale');

		this.callParent([newLocale]);

		this.images.each(function (image) {
			image.data = this.switchTranslations(image.data, newLocale, oldLocale, ['description']);
		}, this);

		this.images.fireEvent('datachanged', this.images);
		this.fireEvent('change', this);
	},

	normalizeData: function (data) {
		var oldLocale = this.localeCode || Sb.Editor.model.get('locale');
		data = this.callParent([data]);

		Ext.each(data.images, function (image, index) {
			data.images[index] = this.switchTranslations(
				image, Sb.Editor.model.get('locale'), oldLocale, ['description']
			);
		}, this);

		return data;
	},

	updateTranslationList: function (locales) {
		this.callParent([locales]);

		this.images.each(function (image) {
			this.clearTranslationList(image.get('translations'), locales);
		}, this);
	},

	unserialize: function (data) {
		var isChanged = false;

		if (Ext.isArray(data.images)) {
			this.images.loadData(data.images);
			delete data.images;
			isChanged = true;
		}

		var isDataChanged = Sb.model.SliderWidget.superclass.unserialize.call(this, data);
		if (isChanged) {
			this.updateIsConfiguredProperties();
			if (!isDataChanged) {
				this.fireEvent('change', this);
			}
		}

		return isDataChanged || isChanged;
	},

	// :TODO: asinko 20120201: See todo to Sb.model.ContactWidget.serialize()
	serialize: function (rootData) {
		var isInitiator	= !rootData; rootData = rootData || {};

		var data = Sb.model.SliderWidget.superclass.serialize.call(this);
		delete data.widgets;
		data.images = [];
		this.images.each(function (image) {
			data.images.push(Ext.apply({}, image.data));
		});

		if (!rootData.widgetList) {
			rootData.widgetList = {};
		}
		if (!rootData.widgetList[this.data.uuid]) {
			rootData.widgetList[this.data.uuid] = data;
		}

		return (isInitiator) ? data : this.data.uuid;
	},

	getResourcesUrl: function () {
		return Sb.model.SliderWidget.superclass.getResourcesUrl.call(this)
			+ '/' + Sb.Config.path.attachmentsFolder + '/Slider/' + this.get('resourcesDir');
	},

	computeIsConfiguredProperties: function () {
		return this.images.getCount() > 0;
	}
});