/**
 * @class Sb.widget.BlogPostEditPanel
 * @extends Sb.widget.WidgetEditTabPanel
 * Class for blog post widget edit panel
 * @constructor
 * @param {Object} config The config object
 */
Ext.define('Sb.widget.BlogPostEditPanel', {
	extend: 'Sb.widget.WidgetEditTabPanel',

	cls: 'sb-widget-edit-panel-blog-post',

	initComponent: function () {
		var items = [];

		if (Sb.Config.widgets.indexOf('commenting') != -1) {
			items.push({
				cls: 'sb-widget-edit-panel-blog-post-comments-panel',
				title: Sb.Locale.get('widgetBlogPostCommentsTabTitle'),
				autoHeight: true,
				layout		: 'form',
				labelWidth	: 275,
				items		: [{
					ref			: '../../isAllowCommentsField',
					xtype		: 'checkbox',
					hideLabel	: true,
					boxLabel	: Sb.Locale.get('widgetBlogPostIsAllowCommentsFieldTitle'),
					scope		: this,
					handler		: function (checkbox, checked) {
						var fn = checked ? 'addCommentingToPost' : 'removeCommentingFromPost';
						Sb.Editor.getController('Blogs')[fn](this.widget.model);
					}
				}],
				listeners: {
					scope: this,
					activate: this.initCommentsTab
				}
			});
		}

		items.push({
			cls: 'sb-widget-edit-panel-blog-post-seo-panel',
			title: Sb.Locale.get('widgetBlogPostSeoTabTitle'),
			autoHeight	: true,
			layout		: 'form',
			labelAlign	: 'top',
			items		: [{
				ref			: '../../slugField',
				xtype		: 'textfield',
				anchor		: '100%',
				fieldLabel	: Sb.Locale.get('widgetBlogPostSlugFieldTitle'),
				maxLength	: 255,
				regex		: /^[a-z0-9\-_]+$/i,
				maskRe		: /[a-z0-9\-_]/i,
				msgTarget	: 'under',
				validator	: function (slug) {
					if (slug === '') {
						this.createAutoSlug();
						return Sb.Locale.get('pageSlugAutoGenerated');
					}
					var page = this.widget.model.ownerPage;
					if (Sb.Config.invalidSlugs.indexOf(slug.toLowerCase()) != -1) {
						return Sb.Locale.get('pageSlugReserved');
					}
					var msg = true;
					page.parentPage.pages.each(function (curPage) {
						if (page != curPage && curPage.data.slug == slug) {
							msg = Sb.Locale.get('pageSlugDuplicate');
							return false;
						}
					});
					return msg;
				}.createDelegate(this),
				listeners	: {
					scope: this,
					valid: function (field) {
						var page = this.widget.model.ownerPage;
						if (field.getValue() !== '' && page.data.slugIsAuto == '1') {
							page.set('slugIsAuto', '0');
						}
					},
					blur: function (field) {
						if (field.getValue() !== '') {
							return;
						}
						this.createAutoSlug();
					}
				}
			}, {
				layout: 'column',
				items: [{
					columnWidth	: 0.5,
					layout		: 'form',
					labelAlign	: 'top',
					items: [{
						ref			: '../../../../descriptionField',
						xtype		: 'textarea',
						fieldLabel	: Sb.Locale.get('widgetBlogPostDescriptionFieldTitle'),
						name		: 'keywords',
						msgTarget	: 'under',
						maxLength	: 255,
						anchor		: '-10'
					}]
				}, {
					columnWidth	: 0.5,
					layout		: 'form',
					labelAlign	: 'top',
					items: [{
						ref			: '../../../../keywordsField',
						xtype		: 'textarea',
						fieldLabel	: Sb.Locale.get('widgetBlogPostKeywordsFieldTitle'),
						name		: 'keywords',
						msgTarget	: 'under',
						maxLength	: 255,
						anchor		: '100%'
					}]
				}]
			}],
			listeners: {
				scope: this,
				activate: this.initSeoTab
			}
		});

		Ext.apply(this, {
			items: items
		});

		Sb.widget.BlogPostEditPanel.superclass.initComponent.apply(this, arguments);
	},

	show: function () {
		this.loadedTabs = {};
		Sb.widget.BlogPostEditPanel.superclass.show.apply(this, arguments);
	},

	hide: function () {
		this.callParent();
		this.widget.model.ownerPage.un('propertychange', this.onPropertyChange, this);
	},

	initCommentsTab: function () {
		this.isAllowCommentsField.setValue(
			Boolean(Sb.Editor.getController('Blogs').findCommentingOnPost(this.widget.model))
		);
		this.reanchor();
	},

	initSeoTab: function () {
		this.reanchor();

		if (this.loadedTabs.seo) {
			return;
		}
		this.loadedTabs.seo = true;

		this.slugField.suspendEvents();
		this.bindModel({
			slug		: 'slugField',
			description	: 'descriptionField',
			keywords	: 'keywordsField'
		}, this.widget.model.ownerPage);
		this.slugField.resumeEvents();

		this.widget.model.ownerPage.on('propertychange', this.onPropertyChange, this);
	},

	onPropertyChange: function (name, value) {
		if (['description', 'keywords'].indexOf(name) !== -1) {
			this[name + 'Field'].setValue(value);
		}
	},

	createAutoSlug: function () {
		var page = this.widget.model.ownerPage;
		Sb.PageService.createSlug(page.get('title'), function (slug) {
			page.set('slugIsAuto', '1');
			page.set('slug', slug);
			page.makeSlugUnique();
			this.slugField.setRawValue(page.get('slug'));
		}, this);
	}
});