/**
 * @class Sb.widget.Commenting
 * @extends Sb.widget.Widget
 * Class for Commenting widget
 * @xtype commenting_widget
 */
Ext.define('Sb.widget.Commenting', {
	extend: 'Sb.widget.Widget',
	xtype: 'commenting_widget',

	cls: 'sb-widget-commenting',

	onRender: function () {
		Sb.widget.Commenting.superclass.onRender.apply(this, arguments);

		if (this.model.isNew) {
			this.model.set('thread', this.model.generateThreadId());
			this.model.save();
		}
	},

	onContainerAdd: function () {
		this.getType().toggle(false);
	},

	onDestroy: function () {
		if (!Sb.Editor.isDemoMode()) {
			this.getType().toggle(true);
		}
		Sb.widget.Commenting.superclass.onDestroy.apply(this, arguments);
	},

	onDocumentClick: function (event) {
		if (Ext.fly(event.target).up('.dsq-popup')) {
			return;
		}
		Sb.widget.Commenting.superclass.onDocumentClick.apply(this, arguments);
	},

	renderContent: function () {
		this.update('<div id="disqus_thread"></div>');
	},

	/**
	 * Build commenting code
	 */
	afterRenderContent: function () {
		window.DISQUS = null;
		var forum = Sb.Editor.model.get('disqusId'),
			locale = this.getLocale();

		window.disqus_developer = 1;
		window.disqus_shortname = forum;
		window.disqus_identifier = this.model.get('thread');
		window.disqus_url = this.model.getUrl() + '?' + this.model.get('thread');
		window.disqus_title = this.model.getTitle();

		window.disqus_config = function () {
			this.language = locale;
			this.callbacks.onReady.push(function () {
				if (Sb.Editor.Ui.editComponent) {
					Sb.Editor.Ui.editComponent.getEditPanel().reanchor();
				}
			});
		};

		if (forum) {
			var requiredItem = {
				url: '//' + forum + '.' + Sb.Config.disqusDomain + '/embed.js',
				force: true, // disqus script run once after loading
				onError: function () {
					this.renderPlaceholder(Sb.Locale.get('disqusForumNotFound', forum));
				}.createDelegate(this)
			};

			Sb.ResourceManager.require([requiredItem], function () {
				if (Sb.Editor.model.isRtl()) {
					Sb.ResourceManager.require(
						[Sb.Config.siteResourcesUrl + '/modules/commenting/rtl.js'],
						function () {
							fixCommentingMarginForRtl();
						}
					);
				}
			}, this);
		}
	},

	/**
	 * Change current disqus forum
	 */
	changeForum: function (forum) {
		Sb.Editor.model.set('disqusId', forum);
		Sb.Editor.model.save();
		this.refreshContent();
	},

	getLocale: function () {
		var locale = Sb.Editor.siteView.getCurrentSiteLocale(),
			lang,
			notExistLocales = ['aa_DJ', 'ak_GH', 'be_BY', 'bn_BD', 'dv_MV', 'dz_BT',
				'fo_FO', 'ha_NE', 'hy_AM', 'iu_GL', 'ka_GE', 'kk_KZ', 'ky_KG', 'lo_LA', 'mn_MN',
				'mt_MT', 'ne_NP', 'ny_MW', 'rw_RW', 'si_LK', 'st_LS', 'sw_CD', 'sw_SO', 'sw_TZ',
				'sw_UG', 'tg_TJ', 'ti_ER', 'to_TO', 'uz_UZ'];

		if (notExistLocales.indexOf(locale) != -1) {
			locale = Sb.Config.fallbackLocale;
		}

		lang = locale.substr(0, 2);

		if (lang == 'sv') {
			return 'sv_SE';
		}

		switch (locale) {
			case 'es_ES':
				return 'es_ES';
			case 'pt_BR':
				return 'pt_BR';
			default:
				return lang;
		}
	}
});