/**
 * @class Sb.widget.ContactEditPanel
 * @extends Sb.widget.WidgetEditTabPanel
 * Class for contact widget edit panel
 * @constructor
 * @param {Object} config The config object
 */
Ext.define('Sb.widget.ContactEditPanel', {
	extend: 'Sb.widget.WidgetEditTabPanel',

	initComponent: function () {
		this.recipientsField = new Ext.form.TextField({
			fieldLabel: Sb.Locale.get('widgetContactRecipientsFieldTitle'),
			width: 510,
			allowBlank: false,
			maxLength: 255,
			vtype: 'emailMulti',
			validationEvent: 'blur',
			msgTarget: 'under',
			ctCls: 'sb-widget-contact-recipients'
		});

		this.subjectField = new Ext.form.TextField({
			fieldLabel	: Sb.Locale.get('widgetContactSubjectFieldTitle'),
			width: 510,
			allowBlank	: false,
			maxLength	: 255,
			msgTarget	: 'under',
			listeners	: {
				scope: this,
				valid: function (field) {
					this.widget.model.unserialize({
						subject: field.getValue()
					});
				}
			}
		});

		this.replyField = new Ext.form.TextArea({
			hideLabel	: true,
			width: 510,
			maxLength	: 255,
			listeners	: {
				scope: this,
				valid: function (field) {
					this.widget.model.unserialize({
						reply: field.getValue()
					});
				}
			}
		});

		this.buttonTextField = new Ext.form.TextField({
			fieldLabel	: Sb.Locale.get('widgetContactButtonTextFieldTitle'),
			width: 510,
			allowBlank	: false,
			maxLength	: 40,
			msgTarget	: 'under',
			listeners	: {
				scope: this,
				valid: function (field) {
					this.widget.model.unserialize({
						buttonText: field.getValue()
					});
				}
			}
		});

		this.captchaDisabledField = new Ext.form.Checkbox({
			hideLabel: true,
			boxLabel: Sb.Locale.get('widgetContactCaptchaEnableFieldTitle'),
			listeners: {
				scope: this,
				check: function (checkbox, checked) {
					this.widget.model.set('captchaEnabled', checked);
				}
			}
		});

		this.fieldTitleField = new Ext.form.TextField({
			xtype: 'textfield',
			fieldLabel: Sb.Locale.get('widgetContactFieldTitleFieldTitle'),
			name: 'title',
			allowBlank: false,
			maxLength: 100,
			anchor: '100%',
			msgTarget: 'under',
			listeners: {
				scope: this,
				valid: function (field) {
					var node = this.fieldsSM.getSelectedNode();
					node.setText(Ext.util.Format.htmlEncode(
						Ext.util.Format.ellipsis(field.getValue(), 28 - node.getDepth() * 2)
					));
					node.attributes.title = field.getValue();
					this.widget.model.updateField(node.id, {
						title: field.getValue()
					});
				}
			}
		});
		this.fieldTypeField = new Ext.form.ComboBox({
			fieldLabel: Sb.Locale.get('widgetContactFieldTypeFieldTitle'),
			name: 'type',
			msgTarget: 'under',
			anchor: '100%',
			store: [
				['textfield', Sb.Locale.get('widgetContactFieldTypeTextfield')],
				['textarea', Sb.Locale.get('widgetContactFieldTypeTextarea')],
				['email', Sb.Locale.get('widgetContactFieldTypeEmail')]
			],
			listeners: {
				scope: this,
				select: function (combo, record) {
					var node = this.fieldsSM.getSelectedNode();
					node.attributes.type = combo.getValue();
					this.widget.model.updateField(this.fieldsSM.getSelectedNode().id, {
						type: combo.getValue()
					});
				}
			}
		});
		this.fieldRequiredField = new Ext.form.Checkbox({
			name: 'required',
			hideLabel: true,
			boxLabel: Sb.Locale.get('widgetContactFieldRequiredFieldTitle'),
			listeners: {
				scope: this,
				check: function (checkbox, checked) {
					var node = this.fieldsSM.getSelectedNode();
					node.attributes.required = checked;
					this.widget.model.updateField(this.fieldsSM.getSelectedNode().id, {
						required: checked
					});
				}
			}
		});

		this.fieldsSM = new Ext.tree.DefaultSelectionModel({
			listeners: {
				scope: this,
				selectionchange: function (sm, node) {
					if (!node) {
						return;
					}
					this.fieldTitleField.setValue(node.attributes.title);
					this.fieldTypeField.setValue(node.attributes.type);
					this.fieldRequiredField.setValue(node.attributes.required);
				}
			}
		});
		this.fields = new Ext.tree.TreeNode();

		this.addFieldButton = new Ext.Button({
			text: Sb.Locale.get('widgetContactAddFieldTitle'),
			tooltip: Sb.Locale.get('widgetContactAddFieldTooltip'),
			iconCls: 'sb-add-icon',
			handler : this.addField.createDelegate(this, [])
		});
		this.deleteFieldButton = new Ext.Button({
			text: Sb.Locale.get('widgetContactDeleteFieldTitle'),
			tooltip: Sb.Locale.get('widgetContactDeleteFieldTooltip'),
			iconCls: 'sb-delete-icon',
			handler : this.deleteField.createDelegate(this)
		});

		Ext.apply(this, {
			cls: 'sb-widget-edit-panel-contact',
			items: [{
				cls: 'sb-widget-contact-settings-panel',
				title: Sb.Locale.get('widgetContactSettingsTabTitle'),
				autoHeight: true,
				layout: 'form',
				labelAlign: 'top',
				defaults: {
					validateOnBlur: true
				},
				items: [
					this.recipientsField,
					{
						xtype: 'box',
						cls: 'sb-editor-box-description',
						html: Sb.Locale.get('widgetContactRecipientsFieldDescription')
					},
					this.subjectField,
					this.buttonTextField,
					this.captchaDisabledField
				]
			}, {
				cls: 'sb-widget-contact-fields-panel',
				title: Sb.Locale.get('widgetContactFieldsTabTitle'),
				autoHeight: true,
				layout: 'column',
				defaults: {
					buttonAlign: 'left'
				},
				items: [{
					width: 300,
					xtype: 'treepanel',
					baseCls: 'sb-pages-tree',
					height: 161,
					autoScroll: true,
					animate: true,
					containerScroll: true,
					enableDD: true,
					lines: false,
					rootVisible: false,
					root: this.fields,
					selModel: this.fieldsSM,
					listeners: {
						scope: this,
						movenode: function (tree, node, oldParent, newParent, index) {
							this.widget.model.insertField(node.id, index);
							node.select();
						},
						startdrag: function (panel, node, e) {
							var el = panel.dragZone.proxy.el;
							el.disableShadow();
							el.setXY(el.getXY());
							var ghost = panel.dragZone.proxy.ghost;
							ghost.addClass('sb-pages-tree-drag-ghost');
							ghost.setWidth(Ext.get(node.ui.wrap).getWidth());
						},
						beforemovenode: function (tree, node, oldParent, newParent, index) {
							return node.getUI().allowMoveToNode(newParent);
						}
					},
					buttons: [this.addFieldButton, this.deleteFieldButton]
				}, {
					width: 210,
					layout: 'form',
					labelAlign: 'top',
					defaults: {
						validateOnBlur: false
					},
					items: [this.fieldTitleField, this.fieldTypeField, this.fieldRequiredField]
				}],
				listeners: {
					scope: this,
					activate: this.initFieldsTab
				}
			}, {
				cls: 'sb-widget-contact-reply-panel',
				title: Sb.Locale.get('widgetContactReplyTabTitle'),
				autoHeight: true,
				layout: 'form',
				labelAlign: 'top',
				items: this.replyField
			}]
		});

		Sb.widget.ContactEditPanel.superclass.initComponent.apply(this, arguments);

		this.tabs.on('tabchange', function () {
			this.reanchor();
		}, this);
	},

	onChange: function () {
		this.updateTranslatableFields();
	},

	updateTranslatableFields: function () {
		var model = this.widget.model;
		this.subjectField.setValue(model.get('subject'));
		this.buttonTextField.setValue(model.get('buttonText'));
		this.replyField.setValue(model.get('reply'));
	},

	show: function () {
		Sb.widget.ContactEditPanel.superclass.show.apply(this, arguments);
		this.recipientsField.setValue(this.widget.model.get('recipients'));
		this.captchaDisabledField.setValue(this.widget.model.get('captchaEnabled'));
		this.updateTranslatableFields();
		this.widget.model.on('change', this.onChange, this);
		this.widget.model.on('changefields', this.onChangeFields, this);
	},

	hide: function () {
		this.callParent();
		this.widget.model.un('change', this.onChange, this);
		this.widget.model.un('changefields', this.onChangeFields, this);
	},

	onChangeFields: function () {
		if (this.fields.rendered) {
			this.initFieldsTab();
		}
	},

	initFieldsTab: function () {
		this.fields.removeAll(true);
		this.fieldAutoId = 0;
		Ext.each(this.widget.model.fields, function (field) {
			var m = field.name.match(/^field(\d+)$/);
			if (m && (parseInt(m[1], 10) > this.fieldAutoId)) {
				this.fieldAutoId = parseInt(m[1], 10);
			}
			this.fields.appendChild(new Ext.tree.TreeNode({
				id: field.name,
				text: Ext.util.Format.htmlEncode(Ext.util.Format.ellipsis(field.title, 26)),
				title: field.title,
				required: field.required,
				type: field.type,
				leaf: true,
				uiProvider: Sb.widget.ContactFieldsTreeNodeUI
			}));
		}, this);

		if (this.fields.firstChild) {
			this.fields.firstChild.select();
		}

		this.deleteFieldButton.setDisabled(this.fields.childNodes.length < 2);
	},

	addField: function (attrs) {
		attrs = Ext.apply({
			name: 'field' + (++this.fieldAutoId),
			title: Sb.Locale.get('widgetContactNewFieldTitle'),
			type: 'textfield',
			required: false
		}, attrs || {});

		this.fields.appendChild(new Ext.tree.TreeNode({
			id: attrs.name,
			text: Ext.util.Format.htmlEncode(Ext.util.Format.ellipsis(attrs.title, 26)),
			title: attrs.title,
			required: attrs.required,
			type: attrs.type,
			leaf: true,
			uiProvider: Sb.widget.ContactFieldsTreeNodeUI
		})).select();

		this.widget.model.addField(attrs);
		this.deleteFieldButton.setDisabled(this.fields.childNodes.length < 2);
	},

	deleteField: function () {
		var nodeForDelete = this.fieldsSM.getSelectedNode();
		if (!nodeForDelete) {
			return;
		}

		var nodeForSelect = nodeForDelete.nextSibling || nodeForDelete.previousSibling;

		this.widget.model.deleteField(nodeForDelete.id);
		nodeForDelete.parentNode.removeChild(nodeForDelete);
		nodeForSelect.select();
		this.deleteFieldButton.setDisabled(this.fields.childNodes.length < 2);
	}
});
