/**
 * @class Sb.widget.ContainerVertical
 * @extends Sb.widget.Container
 * @constructor
 * @param {Object} config The config object
 * @xtype container_vertical_widget
 */
Ext.define('Sb.widget.ContainerVertical', {
	extend: 'Sb.widget.Container',
	xtype: 'container_vertical_widget',

	/**
	 * @cfg {Sb.model.ContainerVerticalWidget} model
	 * The {@link Sb.model.Widget} to bind this Widget to.
	 */

	getId: function () {
		if (!this.id) {
			this.id = this.model.get('region');
		}
		return this.id;
	},

	onRender: function (ct) {
		var region = this.model.get('region'),
			partial = new Sb.PartialTemplate(Sb.Editor.model.data.template.partials.placeholder.replace(
				'{content}',
				Sb.Editor.model.data.template.partials.container
		));
		var background = encodeURIComponent(Sb.Editor.model.design.data.background[region + 'Background']);
		partial.update(ct.dom, {
			'id'	: region,
			'class'	: this.getCssClasses(),
			// :TODO: asinko 20100305: temporary dirty hack for compilation container partial templates
			'items'	: [{content: '<div id="container-tmp-content"></div>'}]
		});
		this.el = ct.first();

		this.contentElement = Ext.get('container-tmp-content').parent();
		this.contentElement.update('');
	},

	/**
	 * Check that css value is not null
	 * @param {String} item
	 */
	notNullCssValue: function (item) {
		return item && item != '0';
	},

	afterRender: function () {
		Sb.widget.ContainerVertical.superclass.afterRender.call(this);

		this.items.each(function (item) {
			item.render(this.contentElement);
		}, this);

		this.onDecorationChange();
		this.onBorderRadiusChange();

		this.mon(Sb.Editor.model.design, {
			scope: this,
			decorationchange: this.onDecorationChange,
			backgroundcolorchange: this.onDecorationChange,
			colorschemechange: this.onBackgroundColorChange,
			borderradiuschange: this.onBorderRadiusChange
		});
		this.mon(this.el, 'contextmenu', Sb.Editor.Ui.onContextMenu, Sb.Editor.Ui, {
			type: 'container',
			region: this.model.data.region
		});
	},

	onDecorationChange: function () {
		this.onBackgroundColorChange();
		this.setStylesForBorder();
		this.setBackground();
	},

	/**
	 * Apply new styles for border
	 */
	setStylesForBorder: function () {
		var name = this.getCurrentBorderName(),
			path = this.getBorderPath(name),
			info = this.getBorderInfo(name),
			height = info.height,
			width = info.width;
		
		this.el.down('div.top').setStyle({
			'height': height + 'px',
			'padding-left': width + 'px',
			'background': 'transparent url(\'' + path + '/images/' + name + '-top-left.png\') top left no-repeat'
		}).down('div').setStyle({
			'height': height + 'px',
			'padding-right': width + 'px',
			'background': 'transparent url(\'' + path + '/images/' + name + '-top-right.png\') top right no-repeat'
		}).down('div').setStyle({
			'height': height + 'px',
			'background': 'transparent url(\'' + path + '/images/' + name + '-top.png\') top left repeat-x'
		});

		this.el.down('div.side').setStyle({
			'padding-left': width + 'px',
			'background': 'transparent url(\'' + path + '/images/' + name + '-left.png\') top left repeat-y'
		}).down('div').setStyle({
			'padding-right': width + 'px',
			'background': 'transparent url(\'' + path + '/images/' + name + '-right.png\') top right repeat-y'
		});

		this.el.down('div.bottom').setStyle({
			'height': height + 'px',
			'padding-left': width + 'px',
			'background': 'transparent url(\'' + path + '/images/' + name + '-bottom-left.png\') bottom left no-repeat'
		}).down('div').setStyle({
			'height': height + 'px',
			'padding-right': width + 'px',
			'background': 'transparent url(\'' + path + '/images/' + name + '-bottom-right.png\') bottom right no-repeat'
		}).down('div').setStyle({
			'height': height + 'px',
			'background': 'transparent url(\'' + path + '/images/' + name + '-bottom.png\') bottom left repeat-x'
		});
	},

	setBackground: function () {
		var background = Sb.Editor.model.design.getDecoration('background', this.model.data.region + 'Background');

		if (Sb.Utils.getBackgroundType(background).value === 'none') {
			return;
		}

		Sb.Editor.siteView.applyContainerBackground(this.el, this.model.get('region'), true);
	},

	/**
	 * Update background color.
	 */
	onBackgroundColorChange: function () {
		var	modelColor = Sb.Editor.model.design.data.backgroundColor[this.model.data.region + 'Color'],
			region = this.model.data.region,
			backgroundColor	= modelColor || 'transparent',
			background = Sb.Editor.model.design.getDecoration('background', region + 'Background'),
			opacity = Sb.Editor.model.design.getDecoration('background', region + 'BackgroundOpacity');

		if (Sb.Utils.getBackgroundType(background).value !== 'none') {
			return;
		}

		if (
			['header', 'content', 'footer'].indexOf(region) > -1
			&& Sb.Editor.model.design.getDecoration('layout', region + 'Layout') == 'elastic-background'
		) {
			backgroundColor	= 'transparent';
		}

		backgroundColor = Sb.Editor.model.design.getColor(backgroundColor);
		if (Ext.ux.util.Color.isValid(backgroundColor)) {
			backgroundColor = Ext.ux.util.Color.hex2rgba(backgroundColor, opacity);
			backgroundColor = 'rgba(' + backgroundColor.join(',') + ')';
		}

		this.el.setStyle({
			backgroundColor: backgroundColor,
			backgroundImage: 'none'
		});
	},

	/**
	 * Update border round.
	 */
	onBorderRadiusChange: function () {
		var borderRadius = Sb.Editor.model.design.getDecoration(
				'borderRadius', this.model.data.region + 'BorderRadius'
			);

		if (Sb.Editor.model.isRtl()) {
			borderRadius = Sb.Utils.swapCssCornersValues(borderRadius);
		}

		if (
			this.model.data.region == 'content'
			&& Sb.Editor.model.design.getDecoration('layout', 'contentLayout') == 'elastic-background'
		) {
			borderRadius = 0;
		}

		this.el.setStyle('borderRadius', borderRadius);
		this.el.child('.container-content-inner').setStyle('borderRadius', borderRadius);
	},

	/**
	 * Prepeare and return base css classes for container
	 *
	 * @return {String} Css classes of container
	 */
	getCssClasses: function () {
		var region			= this.model.data.region,
			cssClasses		= region + ' ' + region + '-color';

		if (/column\d/.test(region)) {
			cssClasses += ' column';
		}

		return cssClasses;
	},

	getLayoutTarget: function () {
		return this.contentElement;
	},

	onDestroy: function () {
		Sb.StyleManager.addStyleSheet('', 'widget-container-vertical', 'corner-' + this.model.data.uuid);
		Sb.widget.ContainerVertical.superclass.onDestroy.call(this);
	},

	/**
	 * Get current border name
	 *
	 * @return {String}
	 */
	getCurrentBorderName: function() {
		return Sb.Editor.model.design.data.border[this.model.data.region + 'Border'];
	},

	/**
	 * Get border meta info by name
	 *
	 * @return {Object}
	 */
	getBorderInfo: function(name) {
		return Sb.Editor.model.data.template.decorations.borders[name];
	},

	/**
	 * Get border path by name
	 *
	 * @return {String}
	 */
	getBorderPath: function (name) {
		return Sb.Config.templatesUrl + '/' +
			encodeURIComponent(Sb.Editor.model.data.template.code) +
			'/decorations/border/' + encodeURIComponent(name);
	}
});