/**
 * @class Sb.widget.EshopBasket
 * @extends Sb.widget.Widget
 * Class for Eshop Basket widget
 * @xtype eshop_basket_widget
 */
Ext.define('Sb.widget.EshopBasket', {
	extend: 'Sb.widget.Widget',
	xtype: 'eshop_basket_widget',

	cls: 'sb-widget-eshop sb-widget-eshop-catalog-basket',

	ecwidWidgetType: 'Minicart',

	ecwidWidgetConfig: {},

	refreshContent: function () {
		Sb.widget.EshopBasket.superclass.refreshContent.apply(this, arguments);
		this.onConfigure();
	},

	onConfigure: function () {
		this.ecwidWidgetConfig = {
			widgetType: 'Minicart',
			id: this.getContentTarget().id,
			widget: this,
			arg: ["layout=" + this.model.data.defaultView]
		};
		Sb.EcwidWidgetsManager.register(this.ecwidWidgetConfig);
	},

	onRemove: function () {
		Sb.EcwidWidgetsManager.unregister(this.ecwidWidgetConfig);
		window.location.hash = '';
	},

	onContainerAdd: function () {
		Sb.EcwidWidgetsManager.refresh();
	},

	/**
	 * Render content for widget`s placeholder
	 */
	renderPlaceholder: function () {
		var link = '<a href="#">' + this.model.getType().getTranslation('inactiveTextLinkText') + '</a>';

		Sb.widget.EshopBasket.superclass.renderPlaceholder.call(this,
			this.model.getType().getTranslation('inactiveText') + ' ' + link
		);

		this.getContentTarget().child('a').on('click', function () {
			var widgetType = Sb.WidgetTypeManager.get('eshopCatalog');
			if (widgetType.getPage().data.identity != Sb.Editor.siteView.currentPage.data.identity) {
				Sb.Editor.siteView.setCurrentPage(widgetType.getPage());
			}
			Sb.EcwidWidgetsManager.getProductBrowserWidget().startEdit();
		}, null, {stopEvent: true});
	},

	onContentLoaded: function () {
		if (this.getEditPanel()) {
			this.getEditPanel().reanchor();
		}
	},

	initEvents: function () {
		Sb.widget.EshopBasket.superclass.initEvents.call(this);

		this.on('remove', this.onRemove, this);
		this.mon(this.model, 'save', function () {}, this);
		this.mon(this.model, 'propertychange', function (key) {
			if (key == 'margin' || key == 'padding') {
				this.onContentLoaded();
				return;
			}

			this.onConfigure();
			Sb.EcwidWidgetsManager.refresh();
		}, this);
	}

});