/**
 * @class Sb.widget.EshopCatalog
 * @extends Sb.widget.Widget
 * Class for EshopCatalog widget
 * @xtype eshop_catalog_widget
 */
Ext.define('Sb.widget.EshopCatalog', {
	extend: 'Sb.widget.Widget',
	xtype: 'eshop_catalog_widget',

	cls: 'sb-widget-eshop-catalog',

	ecwidWidgetType: 'ProductBrowser',

	ecwidWidgetConfig: {},

	ecwidCatalogBasketWidgetConfig: {},

	onRender: function () {
		Sb.widget.EshopCatalog.superclass.onRender.apply(this, arguments);

		this.refreshEl = this.controlsTargetEl.createChild({
			tag: 'span',
			cls: this.baseCls + '-control-item ' + this.baseCls + '-eshop-control-refresh',
			cn: {
				tag: 'span',
				html: Sb.Locale.get('eshopCatalogRefresh')
			}
		});
		this.ecwidControlPanelEl = this.controlsTargetEl.createChild({
			tag: 'span',
			cls: this.baseCls + '-control-item ' + this.baseCls + '-eshop-control-ecwid-control-panel',
			cn: {
				tag: 'span',
				html: Sb.Locale.get('eshopCatalogEcwidControlPanel')
			}
		});
	},

	refreshContent: function () {
		Sb.widget.EshopCatalog.superclass.refreshContent.apply(this, arguments);
		this.onConfigure();
	},

	afterRenderContent: function () {
		this.getContentTarget().createChild({
			id: this.ecwidWidgetType + '-basket-' + this.model.data.uuid,
			cls: 'ecwid-eshop-catalog-basket'
		});
	},

	onContainerAdd: function () {
		Sb.EcwidWidgetsManager.refresh();
		Sb.WidgetTypeManager.toggleButtons();
	},

	onConfigure: function () {
		var gridView = '', listView = '', tableView = '';
		if (this.model.get('viewGridShowInUi')) {
			gridView = 'grid(' + this.model.get('viewGridRows') + ',' + this.model.get('viewGridCols') + ') ';
		}
		if (this.model.get('viewListShowInUi')) {
			listView = 'list(' + this.model.get('viewListRows') + ') ';
		}
		if (this.model.get('viewTableShowInUi')) {
			tableView = 'table(' + this.model.get('viewTableRows') + ') ';
		}

		this.ecwidWidgetConfig = {
			widgetType: this.ecwidWidgetType,
			id: this.getContentTarget().id,
			widget: this,
			arg: [
				"categoriesPerRow=" + this.model.get('categoriesPerRow'),
				"views=" + gridView + listView + tableView,
				"categoryView=" + this.model.get('defaultView'),
				"searchView=" + this.model.get('searchView'),
				"showContactUs=yes"
			]
		};
		Sb.EcwidWidgetsManager.register(this.ecwidWidgetConfig);

		this.ecwidCatalogBasketWidgetConfig = {
				widgetType: 'Minicart',
				id: this.ecwidWidgetType + '-basket-' + this.model.data.uuid,
				widget: this,
				arg: ["layout=MiniAttachToProductBrowser"]
		};
		if (this.model.get('showBasketInCatalog')) {
			Sb.EcwidWidgetsManager.register(this.ecwidCatalogBasketWidgetConfig);
		} else {
			Sb.EcwidWidgetsManager.unregister(this.ecwidCatalogBasketWidgetConfig);
		}
	},

	onCancelEdit: function () {
		Sb.widget.EshopCatalog.superclass.onCancelEdit.call(this);
		if (!Sb.EcwidWidgetsManager.getEshopId()) {
			this.getEditPanel().eshopId.setRawValue('');
		}
	},

	completeEdit: function (explicitCompletion) {
		if (explicitCompletion) {
			var editPanel = this.getEditPanel(),
				formPanel = (editPanel.radioGroup.getValue().inputValue == 'sb-eshop-registration')
					? editPanel.registrationFormPanel
					: editPanel.changeIdFormPanel;

			if (!Sb.EcwidWidgetsManager.getEshopId() && !formPanel.getForm().isValid()) {
				if (editPanel.tabs.getActiveTab() != editPanel.eshopTab) {
					editPanel.tabs.setActiveTab(editPanel.eshopTab);
					formPanel.getForm().isValid(); // Reanchor error messages after changing tab
				}
				editPanel.reanchor();
				return;
			}
		}

		Sb.widget.Contact.superclass.completeEdit.call(this, arguments);
	},

	onRemove: function () {
		Sb.EcwidWidgetsManager.unregister(this.ecwidWidgetConfig);
		Sb.EcwidWidgetsManager.unregister(this.ecwidCatalogBasketWidgetConfig);
		window.location.hash = '';
	},

	/**
	 * Render content for widget`s placeholder
	 */
	renderPlaceholder: function () {
		Sb.widget.EshopCatalog.superclass.renderPlaceholder.call(this,
			this.model.getType().getTranslation('inactiveText',  Sb.Config.general.product)
		);
	},

	onContentLoaded: function () {
		if (this.getEditPanel()) {
			this.getEditPanel().reanchor();
		}
	},

	initEvents: function () {
		Sb.widget.EshopCatalog.superclass.initEvents.call(this);
		this.refreshEl.on('click', function () {
			Sb.EcwidWidgetsManager.refresh(true);
		});
		this.ecwidControlPanelEl.on('click', function () {
			window.open(Sb.Config.ecwidControlPanelUrl);
		});

		this.on('remove', this.onRemove, this);
		this.mon(this.model, 'change', function (model, changedProperties) {
			if (changedProperties.length == 1 && ['margin', 'padding'].indexOf(changedProperties[0]) > -1) {
				this.onContentLoaded();
				return;
			}
			changedProperties.remove('defaultView');
			this.onConfigure();
			if (changedProperties.length) {
				Sb.EcwidWidgetsManager.refresh();
			}
		}, this);
	}
});