/**
 * @class Sb.widget.EshopCatalogEditPanel
 * @extends Sb.widget.WidgetEditTabPanel
 * Class for header widget edit panel
 * @constructor
 * @param {Object} config The config object
 */
Ext.define('Sb.widget.EshopCatalogEditPanel', {
	extend: 'Sb.widget.WidgetEditTabPanel',

	defaultPosition: 't',

	initComponent: function () {

		var catalogView = new Ext.data.ArrayStore({
			fields: ['value', 'display', 'message'],
			data: [['grid', Sb.Locale.get('widgetEshopCatalogGridView'), Sb.Locale.get('widgetEshopCatalogGridViewMessage')],
					['list', Sb.Locale.get('widgetEshopCatalogListView'), Sb.Locale.get('widgetEshopCatalogListViewMessage')],
					['table', Sb.Locale.get('widgetEshopCatalogTableView'), Sb.Locale.get('widgetEshopCatalogTableViewMessage')]]
		});

		Ext.apply(this, {
			cls: 'sb-widget-edit-panel-eshop-catalog',
			items: [{
				ref: '../eshopTab',
				cls: 'sb-widget-eshop-panel',
				title: Sb.Locale.get('widgetEshopTabTitle'),
				helperMessage: Sb.Locale.get('widgetEshopTabMessage', Sb.Config.ecwidControlPanelUrl),
				layout: 'form',
				autoHeight: true,
				items: [{
					ref: '../../radioGroup',
					xtype: 'radiogroup',
					hideLabel: true,
					defaults: {
						xtype: 'radio',
						name: 'select'
					},
					items: [{
						inputValue: 'sb-eshop-registration',
						boxLabel: Sb.Locale.get('eshopNewRegistration')
					}, {
						inputValue: 'sb-eshop-existing',
						checked: true,
						boxLabel: Sb.Locale.get('eshopEnterId')
					}],
					listeners: {
						scope: this,
						change: function (group, radio) {
							this.eshopOptions.layout.setActiveItem(radio.getGroupValue());
							this.reanchor();
						}
					}
				}, {
					ref: '../../eshopOptions',
					layout: 'card',
					xtype: 'container',
					activeItem: 0,
					defaults: {
						xtype: 'form',
						autoHeight: true,
						labelAlign: 'top',
						buttonAlign: 'right'
					},
					items: [{
						id: 'sb-eshop-existing',
						ref: '../../../changeIdFormPanel',
						defaults: {
							allowBlank	: false,
							msgTarget	: 'under',
							anchor		: '100%'
						},
						items: [{
							ref: '../../../../eshopId',
							xtype: 'numberfield',
							minValue	: 1,
							fieldLabel	: Sb.Locale.get('eshopId'),
							listeners	: {
								scope	: this,
								valid	: function (field) {
									this.eshopOwnerEmail.clearInvalid();
									var effect = {
											easing		: 'easeIn',
											duration	: 0.3,
											stopFx		: true,
											useDisplay	: true
										},
										el = this.eshopOwnerEmail.itemCt;
									if (field.getValue() == Sb.EcwidWidgetsManager.getEshopId()) {
										if (el.isVisible()) {
											el.slideOut('t', effect);
										}
									} else if (!el.isVisible()) {
										el.slideIn('t', effect);
									}
								}
							}
						}, {
							ref: '../../../../eshopOwnerEmail',
							xtype: 'textfield',
							vtype: 'email',
							validationEvent: 'blur',
							fieldLabel: Sb.Locale.get('eshopOwnerEmail'),
							ctCls: 'sb-widget-eshop-owner-email'
						}],
						fbar: [{
							ref: '../../../../../btnApplyEshopId',
							text: Sb.Locale.get('eshopApplyButtonEshopId'),
							scope: this,
							handler: function () {
								if (this.changeIdFormPanel.getForm().isValid()) {
									this.applyEshopId(this.eshopId.getValue(), this.eshopOwnerEmail.getValue());
								}
							}
						}]
					}, {
						id: 'sb-eshop-registration',
						ref: '../../../registrationFormPanel',
						defaults: {
							xtype: 'textfield',
							anchor: '100%',
							msgTarget: 'under',
							allowBlank: false
						},
						items: [{
							ref: '../../../../registrationName',
							fieldLabel: Sb.Locale.get('eshopRegistrationName'),
							name: 'name'
						}, {
							ref: '../../../../registrationEmail',
							vtype: 'email',
							validationEvent: 'blur',
							fieldLabel: Sb.Locale.get('eshopRegistrationEmail'),
							name: 'email',
							ctCls: 'sb-widget-eshop-registration-email'
						}, {
							fieldLabel: Sb.Locale.get('eshopRegistrationPassword'),
							inputType: 'password',
							name: 'password'
						}, {
							fieldLabel: Sb.Locale.get('eshopRegistrationPasswordConfirm'),
							inputType: 'password',
							validator: function (value) {
								if (value != this.registrationFormPanel.getForm().findField('password').getValue()) {
									return Sb.Locale.get('eshopPasswordsDifferent');
								}
								return true;
							}.createDelegate(this)
						}],
						fbar: [{
							text: Sb.Locale.get('eshopNewRegistrationOk'),
							scope: this,
							handler: this.registration
						}]
					}]
				}]
			}, {
				ref: '../catalogTab',
				cls: 'sb-widget-eshop-catalog-main-panel',
				title: Sb.Locale.get('widgetEshopCatalogMainTabTitle'),
				helperMessage: Sb.Locale.get('widgetEshopTabMessage', Sb.Config.ecwidControlPanelUrl),
				layout: 'form',
				labelAlign: 'top',
				autoHeight: true,
				items: [{
					ref: '../../categoriesPerRow',
					xtype		: 'numberfield',
					allowBlank	: false,
					minValue	: 1,
					maxValue	: 10,
					width		: 40,
					fieldLabel	: Sb.Locale.get('eshopCatalogCategoriesPerRow')
				}, {
					xtype: 'container',
					cls: 'sb-widget-eshop-catalog-main-panel-view-table',
					layout: 'table',
					layoutConfig: {
						columns: 3
					},
					items: [{
						html: Sb.Locale.get('eshopCatalogViewDefaultLike'),
						colspan: 3
					}, {
						html: ''
					}, {
						html: Sb.Locale.get('eshopCatalogViewRows')
					}, {
						html: Sb.Locale.get('eshopCatalogViewCols')
					}, {
						ref: '../../../gridView',
						xtype: 'radio',
						inputValue: 'grid',
						checked: true,
						name: 'view',
						boxLabel: Sb.Locale.get('widgetEshopCatalogGridView'),
						listeners: {
							scope: this,
							check: this.changeView
						}
					}, {
						ref: '../../../viewGridRows',
						xtype		: 'numberfield',
						allowBlank	: false,
						minValue	: 1,
						maxValue	: 100,
						width		: 40
					}, {
						ref: '../../../viewGridCols',
						xtype		: 'numberfield',
						allowBlank	: false,
						minValue	: 1,
						maxValue	: 10,
						width		: 40
					}, {
						ref: '../../../listView',
						xtype: 'radio',
						inputValue: 'list',
						checked: true,
						name: 'view',
						boxLabel: Sb.Locale.get('widgetEshopCatalogListView'),
						listeners: {
							scope: this,
							check: this.changeView
						}
					}, {
						ref: '../../../viewListRows',
						xtype		: 'numberfield',
						allowBlank	: false,
						minValue	: 1,
						maxValue	: 100,
						width		: 40,
						colspan		: 2
					}, {
						ref: '../../../tableView',
						xtype: 'radio',
						inputValue: 'table',
						checked: true,
						name: 'view',
						boxLabel: Sb.Locale.get('widgetEshopCatalogTableView'),
						listeners: {
							scope: this,
							check: this.changeView
						}
					}, {
						ref: '../../../viewTableRows',
						xtype		: 'numberfield',
						allowBlank	: false,
						minValue	: 1,
						maxValue	: 200,
						width		: 40,
						colspan		: 2
					}]
				}]
			}]
		});
		Sb.widget.EshopCatalogEditPanel.superclass.initComponent.apply(this, arguments);
	},

	registration: function (btn, evt) {
		if (this.registrationFormPanel.getForm().isValid()) {
			btn.disable();
			btn.setIconClass('sb-loading-icon');
			var form = this.registrationFormPanel.getForm(),
				ownerEmail = form.findField('email').getValue();
			Sb.EcwidService.registerNewInstance(form.getFieldValues(), function (result, response) {
				if (result.success) {
					this.eshopId.setValue(result.data.ownerId);
					this.applyEshopId(result.data.ownerId, ownerEmail);
					Sb.Editor.Ui.mainWindow.statusBar.setMessageInfo(Sb.Locale.get('eshopNotificationRegistrationDone'));
				} else {
					if (result.errorType == 'email') {
						this.registrationEmail.markInvalid(result.message);
					} else {
						Sb.Editor.Ui.mainWindow.statusBar.addMessageError(result.message);
					}
				}
				btn.setIconClass('');
				btn.enable();
			}, this);
		}
	},

	applyEshopId: function (eshopId, email) {
		if (eshopId == Sb.EcwidWidgetsManager.getEshopId()) {
			return;
		}
		this.btnApplyEshopId.disable();
		this.btnApplyEshopId.setIconClass('sb-loading-icon');
		Sb.EcwidService.setInstance(eshopId, email, function (result, response) {
			if (result.success) {
				Sb.Editor.model.set('ecwidStoreId', eshopId);
				window.location.hash = '';
				Sb.EcwidWidgetsManager.refresh(true);
				this.tabs.items.item(1).enable();
				this.tabs.setActiveTab(1);
			} else {
				this.eshopOwnerEmail.markInvalid(Sb.Locale.get(result.localeKey, Sb.Config.general.product));
			}
			this.btnApplyEshopId.enable();
			this.btnApplyEshopId.setIconClass('');
		}, this);
	},

	setDefaultView: function (view) {
		var method = view + 'View';
		this[method].setValue(true);
	},

	changeView: function (radio, newValue, oldValue) {
		if (newValue) {
			this.widget.model.set('defaultView', radio.inputValue);
		}
	},

	show: function (widget) {
		Sb.widget.EshopCatalogEditPanel.superclass.show.call(this, widget);
		var data = this.widget.model.data;
		this.bindModel(['categoriesPerRow', 'viewGridCols', 'viewGridRows', 'viewListRows', 'viewTableRows']);

		this.setDefaultView(data.defaultView);
		var eshopId = Sb.EcwidWidgetsManager.getEshopId();
		if (eshopId) {
			this.eshopId.setValue(eshopId);
		}

		if (!Ext.isEmpty(data.registrationName)) {
			this.registrationName.setValue(data.registrationName);
		}
		if (!Ext.isEmpty(data.registrationEmail)) {
			this.registrationEmail.setValue(data.registrationEmail);
		}
		this.registrationFormPanel.getForm().clearInvalid();

		this.eshopOwnerEmail.setValue('');
		this.changeIdFormPanel.getForm().clearInvalid();

		if (!eshopId || Sb.EcwidWidgetsManager.isInvalidEshopId) {
			this.radioGroup.setValue('sb-eshop-registration');
			this.tabs.setActiveTab(0);
			this.tabs.items.item(1).disable();
		} else {
			this.tabs.setActiveTab(1);
			this.tabs.items.item(1).enable();
		}
	}
});