/**
 * @class Sb.widget.NavigationEditPanel
 * @extends Sb.widget.WidgetEditTabPanel
 * Class for navigation widget edit panel
 * @constructor
 * @param {Object} config The config object
 */
Ext.define('Sb.widget.NavigationEditPanel', {
	extend: 'Sb.widget.WidgetEditTabPanel',

	initComponent: function () {
		Ext.apply(this, {
			autoHeight: true,
			cls: 'sb-widget-edit-panel-navigation',
			items: [
				this.createFontColorsTab(), {
					title: Sb.Locale.get('widgetNavigationSpacesTabTitle'),
					autoHeight: true,
					helperMessage: Sb.Locale.get('widgetNavigationSpacesTabMessage'),
					items: {
						ref: '../../spacingTab',
						layout: 'column',
						xtype: 'panel',
						cls: 'sb-widget-navigation-spacing-panel',
						autoHeight: true,
						defaults: {
							xtype: 'form',
							labelWidth: 75,
							autoHeight: true,
							columnWidth: 0.5
						},
						items: [{
							items: [{
								xtype: 'box',
								cls: 'header',
								html: '<div class="image padding"></div><div class="title">' +
									Sb.Locale.get('padding') + '</div>'
							}, {
								xtype: 'expander',
								itemCls: 'sb-bottom-border',
								fieldLabel: Sb.Locale.get('widgetNavigationLinkStateNormal'),
								ref: '../../../../level1Padding',
								minValue: 0,
								maxValue: 100
							}, {
								xtype: 'container',
								layout: 'column',
								itemCls: 'sb-bottom-border',
								fieldLabel: Sb.Locale.get('widgetNavigationLinkStateActive'),
								items: [{
									xtype: 'expander',
									ref: '../../../../../paddingActive',
									minValue: 0,
									maxValue: 100
								}, this.getRevertBtnConfig('paddingActive')]
							}, {
								xtype: 'container',
								layout: 'column',
								fieldLabel: Sb.Locale.get('widgetNavigationLinkStateDropDown'),
								items: [{
									xtype: 'expander',
									ref: '../../../../../level2Padding',
									minValue: 0,
									maxValue: 100
								}, this.getRevertBtnConfig('level2Padding')]
							}]
						}, {
							items: [{
								xtype: 'box',
								cls: 'header',
								html: '<div class="image margin"></div><div class="title">' +
									Sb.Locale.get('margins') + '</div>'
							}, {
								xtype: 'expander',
								itemCls: 'sb-bottom-border',
								fieldLabel: Sb.Locale.get('widgetNavigationLinkStateNormal'),
								ref: '../../../../level1Margin',
								minValue: 0,
								maxValue: 100
							}, {
								xtype: 'container',
								layout: 'column',
								itemCls: 'sb-bottom-border',
								fieldLabel: Sb.Locale.get('widgetNavigationLinkStateActive'),
								items: [{
									xtype: 'expander',
									ref: '../../../../../marginActive',
									minValue: 0,
									maxValue: 100
								}, this.getRevertBtnConfig('marginActive')]
							}, {
								xtype: 'container',
								layout: 'column',
								fieldLabel: Sb.Locale.get('widgetNavigationLinkStateDropDown'),
								items: [{
									xtype: 'expander',
									ref: '../../../../../level2Margin',
									minValue: 0,
									maxValue: 100
								}, this.getRevertBtnConfig('level2Margin')]
							}]
						}]
					}
				}, {
					title: Sb.Locale.get('widgetNavigationDecorationsTabTitle'),
					helperMessage: Sb.Locale.get('widgetNavigationDecorationsTabMessage'),
					items: {
						ref: '../../decorationTab',
						layout: 'form',
						items: [{
							xtype: 'box',
							html: Sb.Locale.get('widgetNavigationDecorationsShadow'),
							fieldLabel: ' '
						}, {
							xtype: 'sbshadowselectbox',
							ref: '../../../level1Shadow',
							itemCls: 'sb-bottom-border',
							fieldLabel: Sb.Locale.get('widgetNavigationLinkStateNormal')
						}, {
							xtype: 'container',
							layout: 'column',
							itemCls: 'sb-bottom-border',
							fieldLabel: Sb.Locale.get('widgetNavigationLinkStateActive'),
							items: [{
								xtype: 'sbshadowselectbox',
								ref: '../../../../shadowActive'
							}, {
								xtype: 'box',
								html: ' ',
								width: 5,
								height: 5
							}, this.getRevertBtnConfig('shadowActive', {
								width: 34,
								height: 34
							})]
						}, {
							xtype: 'container',
							layout: 'column',
							fieldLabel: Sb.Locale.get('widgetNavigationLinkStateDropDown'),
							items: [{
								xtype: 'sbshadowselectbox',
								ref: '../../../../level2Shadow'
							}, {
								xtype: 'box',
								html: ' ',
								width: 5,
								height: 5
							}, this.getRevertBtnConfig('level2Shadow', {
								width: 34,
								height: 34
							})]
						}]
					}
				}, {
				ref: '../levelsTab',
				cls: 'sb-widget-navigation-levels-panel',
				title: Sb.Locale.get('widgetNavigationLevelsTabTitle'),
				layout: 'form',
				autoHeight: true,
				labelAlign: 'top',
				items: {
					xtype: 'combo',
					id: 'navigation-level',
					fieldLabel: Sb.Locale.get('showMenuLevels'),
					itemCls: 'sb-navigation-level-select',
					name: 'bgPlace',
					mode: 'local',
					width: 250,
					editable: false,
					triggerAction: 'all',
					lazyInit: false,
					store: new Ext.data.ArrayStore({
						fields: ['value', 'display', 'message'],
						data: [['1-1', Sb.Locale.get('firstLevel'), Sb.Locale.get('widgetNavigationLevelMessage11')],
									['2-1', Sb.Locale.get('secondLevel'), Sb.Locale.get('widgetNavigationLevelMessage21')],
									['1-2', Sb.Locale.get('firstAndSecondLevel'), Sb.Locale.get('widgetNavigationLevelMessage12')],
									['3-1', Sb.Locale.get('thirdLevel'), Sb.Locale.get('widgetNavigationLevelMessage31')],
									['2-2', Sb.Locale.get('secondAndThirdLevels'), Sb.Locale.get('widgetNavigationLevelMessage22')],
									['1-3', Sb.Locale.get('allLevels'), Sb.Locale.get('widgetNavigationLevelMessage13')]]
					}),
					displayField: 'display',
					valueField: 'value',
					autoSelect: true,
					listeners: {
						scope: this,
						select: function (combo, record, index) {
							var value = combo.getValue().split('-');
							this.widget.model.unserialize({
								level: parseInt(value[0], 10),
								depth: parseInt(value[1], 10)
							});
							this.setHelperMessage(this.levelsTab, record.get('message'));
						}
					}
				}
			}, {
				title: Sb.Locale.get('widgetNavigationBordersAndCordersTabTitle'),
				helperMessage: Sb.Locale.get('widgetNavigationMenuBordersCornersMessage'),
				xtype: 'container',
				ref: '../bordersAndCornersTab',
				layout: 'column',
				autoHeight: true,
				items: [{
					xtype: 'container',
					width: 285,
					items: [{
						layout: 'column',
						cls: 'sb-widget-navigation-border-headers',
						items: [{
							xtype: 'label',
							html: Sb.Locale.get('widgetNavigationBorderTitle'),
							columnWidth: 0.35
						}, {
							xtype: 'label',
							html: Sb.Locale.get('widgetNavigationColorTitle'),
							columnWidth: 0.65
						}]
					},
					this.generateBorderRow(0, 'sb-widget-navigation-border-row sb-bottom-border',
						Sb.Locale.get('widgetNavigationLinkStateNormal'),
						['level1BorderWidth', 'level1BorderColor']),
					this.generateBorderRow(1, 'sb-widget-navigation-border-row sb-bottom-border',
						Sb.Locale.get('widgetNavigationLinkStateActive'),
						['borderWidthActive', 'borderColorActive']),
					this.generateBorderRow(2, 'sb-widget-navigation-border-row',
						Sb.Locale.get('widgetNavigationLinkStateDropDown'),
						['level2BorderWidth', 'level2BorderColor'])
				]}, {
					xtype: 'container',
					width: 220,
					items: [
						{
							layout: 'column',
							cls: 'sb-widget-navigation-corners-headers',
							items: [{
								xtype: 'label',
								html: Sb.Locale.get('widgetNavigationCornersTitle'),
								columnWidth: 0.63
							}, {
								xtype: 'label',
								ref: '../../../../roundingLabel',
								html: Sb.Locale.get('widgetNavigationRoundingTitle'),
								columnWidth: 0.37
							}]
						},
						this.generateCornersRow(
							0,
							'sb-widget-navigation-corners-row sb-bottom-border',
							['level1Corners', 'level1CornersRounding']
						),
						this.generateCornersRow(
							1,
							'sb-widget-navigation-corners-row sb-bottom-border',
							['cornersActive', 'cornersRoundingActive']
						),
						this.generateCornersRow(
							2,
							'sb-widget-navigation-corners-row',
							['level2Corners', 'level2CornersRounding']
						)
					]
				}]
			}]
		});

		Sb.widget.NavigationEditPanel.superclass.initComponent.apply(this, arguments);

		this.key2FieldMap = {
			level1Padding: this.level1Padding,
			paddingActive: this.paddingActive,
			level2Padding: this.level2Padding,
			level1Margin: this.level1Margin,
			marginActive: this.marginActive,
			level2Margin: this.level2Margin,
			level1BorderWidth: this.level1BorderWidth,
			borderWidthActive: this.borderWidthActive,
			level2BorderWidth: this.level2BorderWidth,
			level1BorderColor: this.level1BorderColor,
			borderColorActive: this.borderColorActive,
			level2BorderColor: this.level2BorderColor,
			level1Corners: this.level1Corners,
			cornersActive: this.cornersActive,
			level2Corners: this.level2Corners,
			level1CornersRounding: this.level1CornersRounding,
			cornersRoundingActive: this.cornersRoundingActive,
			level2CornersRounding: this.level2CornersRounding,
			level1Shadow: this.level1Shadow,
			shadowActive: this.shadowActive,
			level2Shadow: this.level2Shadow
		};

		this.masterSlaveDepenencies = {
			level1Margin: ['marginActive', 'level2Margin'],
			level1Padding: ['paddingActive', 'level2Padding'],
			level1BorderWidth: ['borderWidthActive', 'level2BorderWidth'],
			level1BorderColor: ['borderColorActive', 'level2BorderColor'],
			level1Corners: ['cornersActive', 'level2Corners'],
			level1CornersRounding: ['cornersRoundingActive', 'level2CornersRounding'],
			level1Shadow: ['shadowActive', 'level2Shadow']
		};

		Ext.iterate(this.masterSlaveDepenencies, function (master, slaves) {
			Ext.each(slaves, function (slave) {
				this.addSlave(this[master], this[slave]);
			}, this);
		}, this);

		this.makeSiblings([this.level1BorderWidth, this.level1BorderColor]);
		this.makeSiblings([this.borderWidthActive, this.borderColorActive]);
		this.makeSiblings([this.level2BorderWidth, this.level2BorderColor]);
		this.makeSiblings([this.level1Corners, this.level1CornersRounding]);
		this.makeSiblings([this.cornersActive, this.cornersRoundingActive]);
		this.makeSiblings([this.level2Corners, this.level2CornersRounding]);
	},

	getRevertBtnConfig: function (fieldsRef, cfg) {
		if (!Ext.isArray(fieldsRef)) {
			fieldsRef = [fieldsRef];
		}
		cfg = cfg || {};

		return Ext.apply({
			xtype: 'button',
			ref: 'revertButton',
			cls: 'sb-revert-button',
			iconCls: 'sb-revert-icon',
			tabIndex: -1,
			tooltip: Sb.Locale.get('widgetNavigationRevertTooltip'),
			tooltipType: 'title',
			scope: this,
			handler: function () {
				Ext.each(fieldsRef, function (fieldRef) {
					var field = this[fieldRef],
						newValue = field.master.isValid(true) ? field.master.getValue() : field.master.lastValidValue;

					this.addSync(field);
					field.setValue(newValue);
					field[field.master.expanded ? 'expand' : 'collapse']();
					this.widget.model.set(fieldRef, newValue);
				}, this);
			}
		}, cfg);
	},

	showRevertButton: function (field) {
		if (field.ownerCt.revertButton) {
			field.ownerCt.revertButton.addClass('sb-show-revert-button');
		}
	},

	hideRevertButton: function (field) {
		if (field.ownerCt.revertButton) {
			field.ownerCt.revertButton.removeClass('sb-show-revert-button');
		}
	},

	removeSync: function (field) {
		field.on('focus', this.showRevertButton, this);
		field.on('blur', this.hideRevertButton, this);

		field.ownerCt.removeClass('sb-sync');
		field.isSync = false;
		if (field.ownerCt.revertButton) {
			field.ownerCt.revertButton.show();
		}
	},

	addSync: function (field) {
		field.un('focus', this.showRevertButton, this);
		field.un('blur', this.hideRevertButton, this);

		field.ownerCt.removeClass('sb-sync').addClass('sb-sync');
		field.isSync = true;
		if (field.ownerCt.revertButton) {
			field.ownerCt.revertButton.hide();
		}
	},

	/**
	 * Generate font and colors tab
	 *
	 * @return {Ext.Panel}
	 */
	createFontColorsTab: function () {
		/**
		 * Menu font and colors tab
		 *
		 * @type Ext.Panel
		 */
		this.fontColorsTab = new Ext.Panel({
			title: Sb.Locale.get('widgetNavigationFontColorsTabTitle'),
			cls: 'sb-widget-navigation-font-colors-panel',
			autoHeight: true
		});
		this.fontColorsTab.on('render', this.renderFontColorsTab, this, {single: true});
		return this.fontColorsTab;
	},

	/**
	 * Render menu items tab
	 */
	renderFontColorsTab: function () {
		/**
		 * Possible link states
		 *
		 * @type Array
		 * @property linkStates
		 */
		this.linkStates = ['Normal', 'Active', 'DropDown'];

		/**
		 * Menu item alignments
		 *
		 * @type Array
		 * @property alignments
		 */
		this.alignments = ['Left', 'Center', 'Right'];

		var smallMargin = 6,
			smallCellWidth = smallMargin * 12,
			mediumCellWidth	= smallCellWidth * 2,
			conf = {
				smallCellWidth: smallCellWidth,
				mediumCellWidth: mediumCellWidth,
				colorComboWidth: smallCellWidth * 1.5,
				widgetControlCls: 'sb-edit-control sb-edit-control-'
			};

		this.createFontGroup(conf);
		this.createColorsGroup(conf);

		this.on('show', this.bindFontColorsTab, this);
		this.fontColorsTab.on('afterrender', this.bindFontColorsTab, this, {single: true});
		this.setHelperMessage(this.fontColorsTab, Sb.Locale.get('widgetNavigationMenuItemsMessage'));
	},

	createFontGroup: function (conf) {
		var fontGroup = new Sb.form.FieldGroup(),
			fontAlignmentDecorationsGroup;

		// Create font/alignment/decoration group
		fontAlignmentDecorationsGroup = new Sb.form.FieldGroup({
			cls: 'clearfix'
		});

		this.fontFamilySizeGroup = new Sb.form.FontFamilySizePanel({
			fontFamilyWidth: conf.mediumCellWidth,
			fontSizeWidth: conf.smallCellWidth,
			widgetControlCls: conf.widgetControlCls + 'font'
		});
		fontAlignmentDecorationsGroup.add(this.fontFamilySizeGroup);

		// Create font/alignment/decoration group -> alignment group
		this.alignmentGroup = new Sb.form.FieldGroup({
			itemId: 'Alignment',
			layout: 'table',
			itemCls: conf.widgetControlCls + 'alignment',
			fieldLabel: Sb.Locale.get('widgetNavigationMenuAlignment')
		});

		Ext.each(this.alignments, function (alignment) {
			var alignmentButton = new Ext.Button({
				itemId: alignment,
				iconCls: 'x-edit-alignment-' + alignment.toLowerCase(),
				enableToggle: true
			});

			alignmentButton.on('toggle', function (button, pressed) {
				if (!pressed) {
					return;
				}
				this.alignmentGroup.items.each(function (button2) {
					if (!button2.pressed) {
						return;
					}
					if (button2.id == button.id) {
						return;
					}
					button2.toggle(false);
				});
			}, this);

			this.alignmentGroup.add(alignmentButton);
		}, this);
		fontAlignmentDecorationsGroup.add(this.alignmentGroup);

		// Create font/alignment/decoration -> decorations
		this.decorationsGroup = new Sb.form.ColorDecorationPanel({
			itemCls: conf.widgetControlCls + 'decoration',
			colorComboWidth: conf.colorComboWidth,
			widgetControlCls: conf.widgetControlCls,
			children: 'decoration'
		});
		fontAlignmentDecorationsGroup.add(this.decorationsGroup);
		fontGroup.add(fontAlignmentDecorationsGroup);

		this.fontColorsTab.add(fontGroup);

		fontAlignmentDecorationsGroup.toggle('Alignment', false);
		this.fontColorsTab.on('activate', function () {
			var isShowAlignment = !(this.widget.el.up('#column1') || this.widget.el.up('#column2'));
			fontAlignmentDecorationsGroup.toggle('Alignment', isShowAlignment);
		}, this);
	},

	createColorsGroup: function (conf) {
		var colorsGroup = new Sb.form.FieldGroup({
				layout: 'column',
				cls: 'sb-field-group sb-field-group-colors'
			}),
			labels = [],
			linkStatesLabels;

		Ext.each(this.linkStates, function (linkState) {
			var label = new Ext.BoxComponent({
				cls: 'sb-field-color-label',
				html: Sb.Locale.get('widgetNavigationLinkState' + linkState)
			});
			labels.push(label);
		});

		linkStatesLabels = new Sb.form.FieldGroup({
			items: labels
		});

		colorsGroup.add(linkStatesLabels);

		// Create content group -> color group
		this.colorGroup = new Sb.form.FieldGroup({
			columnWidth: 0.7
		});

		Ext.each(this.linkStates, function (linkState, i) {
			var colorCombo = new Sb.form.ColorDropDown({
				itemId: linkState + 'Color',
				fieldLabel: Sb.Locale.get('colorDecorationPanelColor'),
				hideLabel: i !== 0,
				itemCls: conf.widgetControlCls + 'color',
				width: conf.colorComboWidth,
				allowColorPalette: true
			});
			this.colorGroup.add(colorCombo);

			var backgroundCombo = new Sb.form.ColorDropDown({
				itemId: linkState + 'BackgroundColor',
				fieldLabel: Sb.Locale.get('colorDecorationPanelBackgroundArea'),
				hideLabel: i !== 0,
				itemCls: conf.widgetControlCls + 'background-color',
				width: conf.colorComboWidth,
				allowColorPalette: true,
				allowBackground: true,
				allowBlank: true
			});
			this.colorGroup.add(backgroundCombo);
		}, this);
		colorsGroup.add(this.colorGroup);

		this.fontColorsTab.add(colorsGroup);
	},

	generateBorderRow: function (rowNum, cls, rowName, references) {
		var msgTarget = new Ext.Container({
				cls: 'x-form-invalid-msg border-msg-target'
			}),
			rowItems = [{
				xtype: 'label',
				text: rowName,
				width: 78
			}, {
				xtype: 'pixelfield',
				width: 45,
				minValue: 0,
				maxValue: 100,
				msgTarget: msgTarget.getId(),
				ref: '../../../../../' + references[0]
			}, {
				xtype: 'colordropdown',
				width: 106,
				allowColorPalette: true,
				ref: '../../../../../' + references[1]
			}];

		if (rowNum !== 0) {
			rowItems.push(this.getRevertBtnConfig(references));
		}

		return {
			cls: cls,
			items: [{
				layout: 'column',
				items: rowItems
			}, msgTarget]
		};
	},

	generateCornersRow: function (rowNum, cls, references) {
		var msgTarget = new Ext.Container({
				cls: 'x-form-invalid-msg border-msg-target'
			}),
			rowItems = [{
				xtype: 'combo',
				ref: '../../../../../' + references[0],
				width: 122,
				mode: 'local',
				editable: false,
				valueField: 'value',
				displayField: 'label',
				store: {
					xtype: 'arraystore',
					fields: ['value', 'label'],
					data: [
						['', Sb.Locale.get('widgetNavigationNoRounding')],
						['all', Sb.Locale.get('widgetNavigationRoundAll')],
						['top', Sb.Locale.get('widgetNavigationTopTab')],
						['bottom', Sb.Locale.get('widgetNavigationBottomTab')],
						['left', Sb.Locale.get('widgetNavigationLeftTab')],
						['right', Sb.Locale.get('widgetNavigationRightTab')]
					]
				}
			}, {
				xtype: 'pixelfield',
				ref: '../../../../../' + references[1],
				cls: 'x-spinner-field',
				msgTarget: msgTarget.getId(),
				minValue: 0,
				maxValue: 60,
				width: 45
			}];

		if (rowNum !== 0) {
			rowItems.push(this.getRevertBtnConfig(references));
		}

		return {
			cls: cls,
			items: [{
				layout: 'column',
				items: rowItems
			}, msgTarget]
		};
	},

	bindFontColorsTab: function () {
		var model = this.widget.model,
			fontFamily, fontFamilyFn, fontSize, fontSizeFn,
			colorTypes = ['Color', 'BackgroundColor'],
			alignmentFn = function () {
				model.setByLevel('Alignment', Sb.Editor.model.isRtl()
					? Sb.Utils.swapLeftAndRight(this.itemId)
					: this.itemId);
			};

		// Font family
		fontFamily = this.fontFamilySizeGroup.getComponent('FontFamily');
		fontFamilyFn = function () {
			model.setByLevel('FontFamily', this.value);
		};
		fontFamily.setValue(
			model.get('level1FontFamilyIsDependent')
				? Sb.Editor.model.design.get('font').textFont
				: model.getByLevel('FontFamily')
		);
		fontFamily.on('select', fontFamilyFn);
		this.on('hide', function () {
			fontFamily.un('select', fontFamilyFn);
		}, null, {single: true});

		// Font size
		fontSize = this.fontFamilySizeGroup.getComponent('FontSize');
		fontSizeFn = function () {
			model.setByLevel('FontSize', parseInt(this.getRawValue(), 10));
		};
		fontSize.setValue(
			model.get('level1FontSizeIsDependent')
				? Sb.Editor.model.design.get('font').textFontSize
				: model.getByLevel('FontSize')
		);
		fontSize.on('select', fontSizeFn);
		fontSize.on('change', fontSizeFn);
		this.on('hide', function () {
			fontSize.un('select', fontSizeFn);
			fontSize.un('change', fontSizeFn);
		}, null, {single: true});

		// Alignment
		var alignmentValue = Sb.Editor.model.isRtl()
			? Sb.Utils.swapLeftAndRight(model.getByLevel('Alignment'))
			: model.getByLevel('Alignment');
		Ext.each(this.alignments, function (alignmentType) {
			var alignmentBtn = this.alignmentGroup.getComponent(alignmentType);
			if (alignmentType == alignmentValue) {
				alignmentBtn.toggle(true);
			}
		}, this);
		Ext.each(this.alignments, function (alignmentType) {
			var alignmentBtn = this.alignmentGroup.getComponent(alignmentType);
			alignmentBtn.on('toggle', alignmentFn);
			this.on('hide', function () {
				alignmentBtn.un('toggle', alignmentFn);
			}, null, {single: true});
		}, this);

		// Decorations
		Ext.each(this.decorationsGroup.decorations, function (decoration) {
			var modelKey = decoration,
				decorationBtn = this.decorationsGroup.getComponent('Decoration').getComponent(decoration),
				decorationFn = function () {
					model.setByLevel(modelKey, this.pressed);
				};

			decorationBtn.toggle(1 == model.getByLevel(modelKey));
			decorationBtn.on('toggle', decorationFn);

			this.on('hide', function () {
				decorationBtn.un('toggle', decorationFn);
			}, null, {single: true});
		}, this);

		// Link state colors and backgrounds
		Ext.each(this.linkStates, function (linkState) {
			var stateForModel = ('Normal' == linkState || 'DropDown' == linkState) ? '' : linkState,
				level;
			if ('DropDown' == linkState) {
				level = 2;
			} else if ('Normal' == linkState) {
				level = 1;
			}

			Ext.each(colorTypes, function (colorType) {
				var colorModelKey = colorType + stateForModel,
					backgroundModelKey = 'Background' + stateForModel,
					colorCmb = this.colorGroup.getComponent(linkState + colorType),
					initialValue = model.getByLevel(colorModelKey, level),
					changeColor = function () {
						model.setByLevel(colorModelKey, this.value, level);
					},
					changeBackground = function () {
						var background = colorCmb.backgroundValue;
						model.setByLevel(backgroundModelKey, background[background.type] || '', level);
						model.setByLevel('BackgroundStretch' + stateForModel, background.stretch, level);
						model.setByLevel('BackgroundPosition' +  stateForModel, background.position, level);
					};

				colorCmb.setValue(initialValue);
				if (colorModelKey.indexOf('Background') > -1) {
					colorCmb.setBackgroundUploadParams(Sb.Config.baseUrl + '/Upload/Background/', {});
					colorCmb.setBackgroundValue(this.getBackground(backgroundModelKey, level));
				}

				colorCmb.on('change', changeColor);
				this.on('hide', function () {
					colorCmb.un('change', changeColor);
				}, null, {single: true});

				colorCmb.on('backgroundchange', changeBackground);
				this.on('hide', function () {
					colorCmb.un('backgroundchange', changeBackground);
				}, null, {single: true});
			}, this);
		}, this);
	},

	addSlave: function (master, slave) {
		var stateEvents = ['expand', 'collapse'],
			changeEvent = ['sbshadowselectbox', 'combo'].indexOf(master.getXType()) !== -1 ? 'select' : 'change',
			changeSlavesState = function (list, eventName) {
				Ext.each(list, function (slaveField) {
					if (slaveField.isSync) {
						slaveField[eventName]();
					}
				});
			};
		if (!master.slaveList) {
			master.slaveList = [];
			master.on(changeEvent, function (field) {
				if (!field.isValid(true)) {
					return;
				}
				var newValue = field.getValue();
				field.lastValidValue = newValue;

				Ext.each(field.slaveList, function (slave) {
					if (slave.isSync) {
						Ext.iterate(this.key2FieldMap, function (key, field) {
							if (field === slave) {
								this.widget.model.set(key, newValue);
							}
						}, this);
						slave.setValue(newValue);
					}
				}, this);
			}, this);

			if (master instanceof Sb.form.Expander) {
				Ext.each(stateEvents, function (eventName) {
					master.on(eventName, function () {
						changeSlavesState(master.slaveList, eventName);
					});
				});
			}
		}
		master.slaveList.push(slave);
		slave.master = master;
		slave.on(changeEvent, function () {
			this.refreshLinks(master);
		}, this);

		if (master instanceof Sb.form.Expander) {
			Ext.each(stateEvents, function (eventName) {
				slave.on(eventName, function () {
					if (eventName === 'expand') {
						slave.ownerCt.addClass('sb-expand');
					} else {
						slave.ownerCt.removeClass('sb-expand');
					}

					if (slave.isSync) {
						changeSlavesState(master.slaveList, eventName);
						master[eventName]();
					}
				});
			});
		}
	},

	makeSiblings: function (siblings) {
		Ext.each(siblings, function (field) {
			field.siblings = [];
			Ext.each(siblings, function (sibling) {
				if (sibling !== field) {
					field.siblings.push(sibling);
				}
			});
		}, this);
	},

	refreshLinks: function (masterField) {
		var slaveMatchMaster = function (slave) {
				return slave.master.getValue() === slave.getValue();
			};
		Ext.each(masterField.slaveList, function (slaveField) {
			this.addSync(slaveField);
			if (!slaveMatchMaster(slaveField)) {
				this.removeSync(slaveField);
			} else if (slaveField.siblings) {
				Ext.each(slaveField.siblings, function (sibling) {
					if (!slaveMatchMaster(sibling)) {
						this.removeSync(sibling);
					}
				}, this);
			}
		}, this);
	},

	changeCornersType: function (key) {
		var field;
		if (['level1Corners', 'cornersActive', 'level2Corners'].indexOf(key) !== -1) {
			field = this[key];
			Ext.each(field.siblings, function (sibling) {
				sibling[this.widget.model.get(key) ? 'show' : 'hide']();
			}, this);

			if (this.level1CornersRounding.hidden && this.level2CornersRounding.hidden
				&& this.cornersRoundingActive.hidden && !this.roundingLabel.hidden) {
				this.roundingLabel.hide();
			} else if (this.roundingLabel.hidden) {
				this.roundingLabel.show();
			}
		}
	},

	show: function (widget) {
		Sb.widget.NavigationEditPanel.superclass.show.call(this, widget);
		var data		= this.widget.model.data,
			value		= data.level + '-' + data.depth,
			selectBox	= Ext.getCmp('navigation-level'),
			store		= selectBox.store,
			record		= store.getAt(store.find('value', value));
		selectBox.setValue(value);
		this.setHelperMessage(this.levelsTab, record.get('message'));
		this.bindModel(this.key2FieldMap);

		Ext.iterate(this.masterSlaveDepenencies, function (key) {
			this[key].lastValidValue = this.widget.model.get(key);
		}, this);

		this.cascade(function (cmp) {
			if (cmp.getXType() === 'expander') {
				cmp.collapse();
			}
		});

		widget.model.on('propertychange', this.changeCornersType, this);
		Ext.each(['level1Corners', 'cornersActive', 'level2Corners'],  this.changeCornersType, this);

		this.cascade(function (field) {
			if (field.slaveList) {
				this.refreshLinks(field);
			}
		}, this);
	},

	hide: function () {
		this.widget.model.un('propertychange', this.changeCornersType, this);
		Sb.widget.NavigationEditPanel.superclass.hide.call(this);
	},

	/**
	 * Returns background properties
	 *
	 * Fork of Sb.model.SiteDesign.getBackground()
	 * @see src/htdocs/js/Sb/model/SiteDesign.js line 502
	 *
	 * @param {String} key
	 * @param {Number} level
	 * @returns {Object}
	 */
	getBackground: function (key, level) {
		var model = this.widget.model,
			background = model.getByLevel(key, level),
			type = Sb.Utils.getBackgroundType(background).value;

		return {
			type: type,
			gradient: type === 'gradient' ? background : 'background10-black',
			pattern: type === 'pattern' ? background : 'background1',
			position: model.getByLevel('BackgroundPosition', level) || 'top left',
			stretch: type === 'pattern' ? model.getByLevel('BackgroundStretch', level) : 'tile'
		};
	}
});