/**
 * @class Sb.widget.Sharethis
 * @extends Sb.widget.Widget
 * Class for Share This widget
 * @constructor
 * @param {Object} config The config object
 * @xtype sharethis_widget
 */
Ext.define('Sb.widget.Sharethis', {
	extend: 'Sb.widget.Widget',
	xtype: 'sharethis_widget',

	cls: 'sb-widget-sharethis',

	onRender: function () {
		Sb.widget.Sharethis.superclass.onRender.apply(this, arguments);

		Sb.ResourceManager.require([{
			type: 'js',
			url: '//s7.addthis.com/js/250/addthis_widget.js',
			onLoad: function () {
				addthis.init();
			}
		}], function () {
			this.refresh();
		}, this);
	},

	refresh: function () {
		this.widgetContentEl.update(Sb.widget.Sharethis.appearancePresets[this.model.get('appearance')].code);
		this.widgetContentEl.select('.addthis_toolbox').each(function (el) {
			el.select('.addthis_button_tweet').each(function (link) {
				link.dom.className = 'at300b';
				link.createChild({
					tag: 'iframe',
					scrolling: 'no',
					frameborder: '0',
					allowtransparency: 'true',
					src: '//platform.twitter.com/widgets/tweet_button.html?count=horizontal&lang=' +
						addthis_config.ui_language + '&url=' + addthis_share.url,
					style: 'width: 110px; height: 20px;'
				});
			}, this);
			addthis.toolbox(el.dom);
		}, this);
		this.widgetContentEl.select('.addthis_button').each(function (el) {
			addthis.button(el.dom);
		}, this);

		if (Sb.Editor.Ui.editComponent == this) {
			this.getEditPanel().reanchor();
		}
	},

	initEvents: function () {
		Sb.widget.Sharethis.superclass.initEvents.call(this);
		this.mon(this.model,	'propertychange',	function (key, value, oldValue) {
			if (key == 'appearance') {
				this.refresh();
			}
		}, this);
	},

	onDocumentClick: function (event) {
		if (event.getTarget('.x-menu')
			|| event.getTarget('#at16lb')
			|| event.getTarget('#at16pib')
		) {
			return;
		}

		Sb.widget.Sharethis.superclass.onDocumentClick.call(this, event);
	},

	onContentElementClick: function (event, target) {
		if (event.getTarget('a[class~="addthis_button"]')
			|| event.getTarget('a[class~="at300b"]')
			|| event.getTarget('a[class~="at300m"]')
		) {
			return;
		}

		Sb.widget.Sharethis.superclass.onContentElementClick.apply(this, arguments);
	}
});