/**
 * @class Sb.widget.WidgetEditTabPanel
 * @extends Sb.widget.WidgetEditPanel
 * Base class for all widget edit panels with tabs
 * @constructor
 * @param {Object} config The config object
 */
Ext.define('Sb.widget.WidgetEditTabPanel', {
	extend: 'Sb.widget.WidgetEditPanel',

	initComponent: function () {
		this.items = {
			xtype: 'tabpanel',
			autoHeight: true,
			items: this.items
		};

		this.cls += ' tabs';

		Sb.widget.WidgetEditTabPanel.superclass.initComponent.apply(this, arguments);

		this.tabs = this.items.item(0);
		this.tabs.frame = true;

		this.tabs.on('beforetabchange', function (panel, tab) {
			if (panel.rendered) {
				var focusEl = panel.el.child('.x-tab-strip-active a.x-tab-right');
				if (focusEl) {
					focusEl.focus();
				}
			}
		}, this);

		this.tabs.on('tabchange', function (panel, tab) {
			this.setHelperMessage(tab, tab.helperMessage);
		}, this);
	},

	afterRender: function () {
		Sb.widget.WidgetEditTabPanel.superclass.afterRender.call(this);

		var hw = this.tabs.header.dom.offsetWidth;
		var	lis = this.tabs.strip.query('li:not(.x-tab-edge)');
		var	len = lis.length, i;
		for (i = 0; i < len; i++) {
			hw -= lis[i].offsetWidth + 2;
		}
		var delta = Math.floor(hw / len);
		for (i = 0; i < len - 1; i++) {
			lis[i].style.width = (lis[i].offsetWidth + delta) + 'px';
			hw -= delta;
		}
		lis[len - 1].style.width = (lis[len - 1].offsetWidth + hw + 2) + 'px';

		this.helperEl = this.tabs.ft.first().first().createChild({
			cls: 'sb-widget-helper',
			cn: {
				cls: 'sb-widget-helper-wrap',
				cn: {
					cls: 'sb-widget-helper-inner',
					cn: {
						cls: 'sb-widget-helper-body'
					}
				}
			}
		});
		this.helperEl.setVisibilityMode(Ext.Element.DISPLAY);
		this.helperBodyEl = this.helperEl.child('.sb-widget-helper-body');
	},

	/**
	 * Toggles display of helper message
	 * @param {Boolean} isShow
	 */
	toggleHelperMessage: function (isShow) {
		this.helperEl[isShow ? 'show' : 'hide']();
		this.reanchor();
	},

	/**
	 * Sets helper message for tab
	 * @param {Ext.Panel} tab
	 * @param {String} [message]
	 */
	setHelperMessage: function (tab, message) {
		tab.helperMessage = message || '';
		this.helperBodyEl.update(tab.helperMessage);
		this.toggleHelperMessage(tab.helperMessage);
	},

	show: function (widget) {
		Sb.widget.WidgetEditTabPanel.superclass.show.call(this, widget);

		var firstTab = this.tabs.items.get(0);
		if (firstTab == this.tabs.getActiveTab()) {
			this.tabs.activeTab.fireEvent('activate', this.tabs.getActiveTab());
		} else {
			this.tabs.setActiveTab(firstTab);
		}
	}
});
