/**
 * @class Sb.widgetType.Video
 * @extends Sb.widgetType.Base
 */
Ext.define('Sb.widgetType.Video', {
	extend: 'Sb.widgetType.Base',

	allowedColors: '',

	constructor: function (name) {
		Sb.widgetType.Video.superclass.constructor.call(this, name);
		Ext.each(this.providersSource, function (provider, i) {
			this.providersSource[i] = provider;
			provider.isUpload	= Ext.isDefined(provider.isUpload) ? provider.isUpload : true;
			provider.isKnown	= Ext.isDefined(provider.isKnown) ? provider.isKnown : true;
			provider.videoRe	= new RegExp(provider.videoRe);
			provider.template	= new Ext.Template(
				'<object width="{width}" height="{height}">',
					'<param name="movie" value="', provider.movie, '"></param>',
					'<param name="allowfullscreen" value="true"></param>',
					'<param name="allowscriptaccess" value="always"></param>',
					'<param name="wmode" value="opaque"></param>',
					'<embed width="{width}" height="{height}" type="application/x-shockwave-flash" ',
						'src="', provider.movie, '" ',
						'allowfullscreen="true" ',
						'allowscriptaccess="always" ',
						'wmode="opaque"></embed>',
				'</object>' , {compiled: true});
		}, this);
	},

	/**
	 * Source of currently supported video hosting providers: their labels, regexps and templates
	 *
	 * @type Array
	 * @property providersSource
	 */
	providersSource: [
		{label: 'YouTube',		host: 'youtube.com',		videoRe: '[\\?&]v=([^&]+)',
		width: 480,				height: 385,
		movie: '//www.youtube.com/v/{videoId}?hl={lang}&amp;rel=0&amp;fs=1'},
		{label: 'YouTube',		host: 'youtu.be',		videoRe: 'youtu\\.be\\/(.+)',
		width: 480,				height: 385,
		movie: '//www.youtube.com/v/{videoId}?hl={lang}&amp;rel=0&amp;fs=1'},
		{label: 'Vimeo',		host: 'vimeo.com',			videoRe: 'vimeo\\.com\\/([0-9]+)',
		width: 400,				height: 225,
		movie: '//vimeo.com/moogaloop.swf?clip_id={videoId}&amp;server=vimeo.com&amp;show_title=1&amp;' +
		'show_byline=0&amp;show_portrait=0&amp;color=&amp;fullscreen=1&amp;autoplay=0&amp;loop=0'},
		{label: 'MySpace',		host: 'myspace.com',		videoRe: 'myspace\\.com\\/.*video.+\\/([0-9]+)\\/?$',
		width: 425,				height: 360,
		movie: '//mediaservices.myspace.com/services/media/embed.aspx/m={videoId},t=1,mt=video'},
		{label: 'Dailymotion',	host: 'dailymotion.com',	videoRe: 'dailymotion\\.com/video/([^\\/]+)',
		width: 480,				height: 360,
		movie: '//www.dailymotion.com/swf/video/{videoId}?additionalInfos=0'},
		{label: 'Google Video',	host: 'video.google.com',	videoRe: '[#\\?]docid=([0-9\\-]+)[^0-9]*$',
		width: 400,				height: 326,
		movie: '//video.google.com/googleplayer.swf?docid={videoId}&amp;hl={lang}&amp;fs=true', isUpload: false}
	],

	/**
	 * Gets current supported video hosting providers
	 *
	 * @param {Object} [filter] Filter for providers
	 * @returns Array
	 */
	getProviders: function (filter) {
		if (undefined === filter) {
			return this.providersSource;
		}
		var result = [];
		Ext.each(this.providersSource, function (provider) {
			if (filter.isUpload != provider.isUpload) {
				return;
			}
			result.push(provider);
		});
		return result;
	}
});