/*!
 * Ext.ux.IconCombo Extension Class
 * @class Ext.ux.IconCombo
 * @extends Ext.form.ComboBox
 * @constructor
 * @param {Object} config Configuration options
 */
Ext.define('Ext.ux.IconCombo', {
	extend: 'Ext.form.ComboBox',
	xtype: 'iconcombo',

	initComponent: function () {
		var tpl = '<tpl for="."><div class="x-combo-list-item x-icon-combo-item';
		if (this.iconClsField) {
			tpl += ' {' + this.iconClsField + '}"';
		}
		if (this.iconStyleField) {
			tpl += '" style="{' + this.iconStyleField + '}"';
		}
		tpl += '>{' + this.displayField + '}</div></tpl>';

		// apply config
		Ext.apply(this, Ext.apply(this.initialConfig, { tpl: tpl }));

		// call parent initComponent
		Ext.ux.IconCombo.superclass.initComponent.apply(this, arguments);
	},

	onRender: function (ct, position) {
		// call parent onRender
		Ext.ux.IconCombo.superclass.onRender.apply(this, arguments);

		// adjust styles
		this.wrap.applyStyles({position: 'relative'});
		this.el.addClass('x-icon-combo-input');

		// add div for icon
		this.icon = Ext.DomHelper.append(this.wrap, {
			tag: 'div',
			className: 'x-icon-combo-icon',
			style: 'position: absolute;'
		});
	},

	afterRender: function () {
		Ext.ux.IconCombo.superclass.afterRender.apply(this, arguments);
		if (undefined !== this.value) {
			this.setValue(this.value);
		}
	},

	refreshIcon: function () {
		var rec = this.store.query(this.valueField, this.getValue()).itemAt(0);
		if (rec && this.icon) {
			if (this.iconClsField) {
				this.icon.className = 'x-icon-combo-icon ' + rec.get(this.iconClsField);
			}
			if (this.iconStyleField) {
				this.icon.className = 'x-icon-combo-icon';
				Ext.DomHelper.applyStyles(this.icon, rec.get(this.iconStyleField));
			}
		}
	},

	setValue: function (value) {
		Ext.ux.IconCombo.superclass.setValue.call(this, value);
		this.refreshIcon();
	},

	clearValue: function () {
		Ext.ux.IconCombo.superclass.clearValue.call(this);
		if (this.icon) {
			this.icon.className = 'x-icon-combo-icon';
			Ext.DomHelper.applyStyles(this.icon, 'position: absolute;');
		}
	}
});