/**
 * @class Ext.ux.uploader.Uploader
 * @extends Ext.util.Observable
 * @constructor
 */
Ext.define('Ext.ux.uploader.Uploader', {
	extend: 'Ext.util.Observable',

	constructor: function (config) {
		this.url = Ext.urlAppend(config.url, Ext.urlEncode(config.params || {}));
		Ext.ux.uploader.Uploader.superclass.constructor.call(this, config);
	},

	/**
	 * @returns {XMLHttpRequest}
	 */
	createXhr: function () {
		var xhr = new XMLHttpRequest();
		xhr.open("POST", this.url, true);
		xhr.setRequestHeader("X-Requested-With", "XMLHttpRequest");
		xhr.setRequestHeader("Cache-Control", "no-cache");
		return xhr;
	},

	/**
	 * @param {Ext.ux.uploader.Item} item
	 */
	uploadItem: function (item) {
		var xhr = this.createXhr(),
			formData = new FormData();

		formData.append('file', item.getFileObject());
		xhr.onreadystatechange = this.stateChangeHandler.createDelegate(this, [xhr, item]);
		xhr.send(formData);
	},

	/**
	 * @param {XMLHttpRequest} xhr
	 * @param {Ext.ux.uploader.Item} item
	 */
	stateChangeHandler: function (xhr, item) {
		// See https://developer.mozilla.org/en-US/docs/DOM/XMLHttpRequest#Properties
		if (xhr.readyState === 4) {
			if (xhr.status === 200) {
				this.onUploadSuccess(xhr, item);
			} else {
				this.onUploadFailure(xhr, item);
			}
		}
	},

	onUploadSuccess: function (response, item) {
		var responseJson,
			info = {
				success: false,
				message: '',
				result: {},
				response: response
			};

		try {
			responseJson = Ext.decode(response.responseText);
		} catch (ex) {}

		if (responseJson) {
			Ext.apply(info, {
				success: responseJson.success || info.success,
				message: responseJson.message || info.message,
				result: responseJson
			});
		}

		if (info.success) {
			this.fireEvent('uploadsuccess', item, info);
		} else {
			this.fireEvent('uploadfailure', item, info);
		}
	},

	onUploadFailure: function (response, item) {
		var info = {
			success: false,
			message: response.statusText,
			response: response
		};

		this.fireEvent('uploadfailure', item, info);
	}

});