<?php
/**
 * Published site environment checker script
 *
 * @since 2006/07/05
 * @package Web
 * @copyright (c) 2004-2014. Parallels IP Holdings GmbH. All rights reserved.
 */

$serverInfo = array(
	'phpMinVersion' => array(
		version_compare(PHP_VERSION, '5.2.0', '>='), // result check
		true, // visible for user
		'The PHP version on the server must be 5.2.0 or later', // description
	),
	'isApacheServer' => array(
		substr(php_sapi_name(), 0, 6) == 'apache',
		false,
		''
	),
	'isWindows' => array(
		strtoupper(substr(PHP_OS, 0, 3)) == 'WIN',
		false,
		'',
	),
	'mbstring' => array(
		extension_loaded('mbstring'),
		true,
		'Multibyte string functions (mbstring extension)',
	),
	'session' => array(
		extension_loaded('session'),
		true,
		'Session handling functions (session extension)',
	),
);

if (isset($unique)) {
	$serverInfo['unique'] = array(
		$unique,
		false,
		'',
	);
}

if (isset($asJSON) && $asJSON) {
	$result = array();
	foreach ($serverInfo as $name => $r) {
		$result[$name] = $r[0];
	}
	echo json_encode($result);
} else {
	$css = <<<CSS
#result {
	width: 480px;
	margin: 0 auto;
}
table {
	width: 100%;
	background-color: #E6ECFF;
	border-collapse: collapse;
}

th, td {
	border: 1px solid #BFCFFF;
	padding: 0.2em;
}

th {
	background-color: #CCD9FF;
	text-align: left;
}
.ok, .fail {
	text-align: center;
}
.ok {
	background-color: #60BF60;
}
.fail {
	background-color: #FF8080;
}
CSS;

	echo "<html><head><title></title><style type='text/css'>{$css}</style></head><body><div id='result'><table>";
	echo "<h2>Presence Builder</h2>";
	echo "<h4>Information about the server for publication</h4>";
	echo "<tr><th>Param</th><th>Result</th><th>Description</th></tr>";

	foreach ($serverInfo as $name => $info) {
		if ($info[1]) {
			$status = $info[0] ? 'ok' : 'fail';
			echo "<tr><td>$name</td><td class='$status'>$status</td><td>$info[2]</td></tr>";
		}
	}

	echo '</div></table></body></html>';
}