<?php
# formating text styles can see http://pear.php.net/package/Console_Color

/**
 * Get real string length without stylecodes
 * @param string $str
 */
function strLength($str) {
	return strlen(SB_View_Filter_ConsoleColor::removeStyle($str));
}

/**
 * create string fixed length
 * @param string $str
 * @param integer $length
 */
function strToFixLength($str, $length) {
	$strLength = strlen(SB_View_Filter_ConsoleColor::removeStyle($str));
	$emptyLength = $length - $strLength;
	if ($emptyLength > 0) {
		$str .= str_repeat(' ', $emptyLength);
	}
	return $str;
}

/**
 * Creates inner message as a raw array of lines
 *
 * @param Exception $e
 * @param boolean $isShowTrace
 * @return array
 */
function createInnerMessage(Exception $e, $isShowTrace = false) {
	$m = array();
	$m[] = '%rError:%n%r%_ ' . $e->getMessage() . '%n';
	if (!$isShowTrace) {
		return $m;
	}
	$m[] = '%r File:%n%r%_ ' . $e->getFile() . '%n';
	$m[] = '%r Line:%n%r%_ ' . $e->getLine() . '%n                                                                                            ';
	$m[] = '';
	$m[] = '%_Stack trace:%n';
	foreach($e->getTrace() as $number => $info) {
		$info = array_merge(array(
			'file' => '',
			'line' => '',
			'class' => '',
			'type' => '',
			'function' => '',
		), $info);
		$m[] = '';
		$m[] = ' %_' . $number . ')%n %g' . $info['file'] . '%n in line %m' . $info['line'] . '%n';
		$arguments = array();
		if (isset($info['args'])) {
			foreach ($info['args'] as $arg) {
				if (is_array($arg)) {
					$arguments[] = 'Array';
				} elseif (is_object($arg)) {
					$arguments[] = 'Object(' . get_class($arg) . ')';
				} elseif (is_string($arg)) {
					$arguments[] = '"' . $arg . '"';
				} else {
					$arguments[] = $arg;
				}
			}
		}
		$m[] = '     %c' . $info['class'] . $info['type'] . $info['function'] . '(%n%y' . implode(', ', $arguments) . '%n%c)%n';
	}
	return $m;
}

/**
 * Wraps message as an array of lines and create new array of lines from it with respect to leading whitespace,
 * also pads whitespace to the end
 *
 * @param array $lines
 * @param integer $maxWidth
 * @return array
 */
function wrapMessage(array $lines, $maxWidth = 120) {
	$m = array();
	foreach ($lines as $line) {
		$line = rtrim($line);
		$trimmedLine = ltrim($line);
		$padding = strLength($line) - strLength($trimmedLine);
		$whitespace = substr($line, 0, $padding);
		$wrappedLines = explode(PHP_EOL, wordwrap($trimmedLine, $maxWidth - $padding, PHP_EOL, true));
		foreach ($wrappedLines as $wrappedLine) {
			$m[] = strToFixLength($whitespace . $wrappedLine, $maxWidth);
		}
	}
	return $m;
}

/**
 * Adds border around the lines by modifing them
 *
 * @param array $lines
 * @param string $horizontalChar
 * @param string $verticalChar
 * @return array
 */
function addBorder(array $lines, $horizontalChar = '%r=%n', $verticalChar = "%r#%n") {
	$maxWidth = 0;
	foreach ($lines as $line) {
		$lineWidth = strLength($line);
		$maxWidth  = ($lineWidth > $maxWidth) ? $lineWidth : $maxWidth;
	}
	$horizontalExtremeChar = ' ';
	$horizontalLine = $horizontalExtremeChar . str_repeat($horizontalChar, $maxWidth + 2) . $horizontalExtremeChar;
	$m = array();
	foreach ($lines as $line) {
		$m[] = $verticalChar . ' ' . str_pad($line, $maxWidth) . ' ' . $verticalChar;
	}
	array_push($m, $horizontalLine);
	array_unshift($m, $horizontalLine);
	return $m;
}

/**
 * Gets text from provided array of lines
 * @return string
 */
function convertLinesToText(array $lines) {
	return implode(PHP_EOL, $lines) . PHP_EOL;
}

echo convertLinesToText(
	addBorder(
		wrapMessage(
			createInnerMessage(
				$this->exception,
				SB_Config::getInstance()->getDebug('debug')
			)
		)
	)
);
